#!/usr/bin/env python
# -*- coding: utf-8 -*-

import pymongo
import logging
import json
import fnmatch
import re
try :
    import solr
except ImportError:
    pass
import uuid
from clfcfg import CLFCfg
import time
import os
import sqlite3
import pytz
import datetime
import calendar
import clf_datetime
from statDef import *
import collections
import re
import itertools
import reportDef
try :
    from statDef import facet_index
except ImportError:
    facet_index ={}
from common_id import summary_map

class DB(object):
    '''Interface class for DB.'''
    def __init__(self):
        pass

    def connect(self):
        pass

    def insert(self, coll, data):
        pass

    def update(self, coll, filter, data):
        pass

    def find(self, coll, filter, result, start=0, rows=10):
        pass

    def remove(self, coll, filter):
        pass

class MongoDB(DB):
    '''MongoDB implementation.'''
    def __init__(self):
        CLFCfg.init()
        self.MongoDBUri = CLFCfg.CLF_DB_MONGODB_URI
        self.conn = None
        self.db = None

    def __del__(self):
        if (not (self.conn is None)):
            self.conn.disconnect()

    def __toDBFormat(self, filter):
        '''Filters list only support 'AND', for example:
            [ {'col':'column1', 'op':'<', 'value':'2'},
              {'col':'column2', 'op':'eq', 'value':['test*']},
              ...
            ]
        It supports numeric filter and string filter.
        'op' of numeric filter could be <, >, =, <=, >=.
        'op' of string filter could be 'eq' (equal) and 'neq' (not equal).
        'value' of string filter support wildcards '*' when 'op' is 'eq', for example, '*test*'
        '''
        query = {}
        ops = {'<': '$lt', '>': '$gt', '<=': '$lte', '>=': '$gte'}
        for cond in filter:
                col = cond['col']
                op = cond['op']
                value = cond['value']

                # change 'value' to 'list'
                if op in ('==', '!=', 'eq', 'neq'):
                    if not isinstance(value, list):
                        value = [value]

                # for '<', '>', '<=', '>=' conditions
                if op in ops:
                    if query.has_key(col):
                        query[col].update({ops[op]:value})
                    else:
                        query.update({col:{ops[op]:value}})
                    continue

                wildcard = False    #wildcard doesn't support 'neq'
                if op == 'eq':
                    for fv in value:
                        if '*' in fv:
                            wildcard = True
                            break
                    if wildcard:
                        if len(value) == 1:
                            regex = fnmatch.translate(value[0])
                            regex = re.sub(r'\\Z\(\?ms\)$', '',regex)
                        else:
                            regex = ''
                            for fv in value:
                                r = fnmatch.translate(fv)
                                r = re.sub(r'\\Z\(\?ms\)$', '',r)
                                regex += r + '|'
                            regex = regex.rstrip('|')

                        query.update({col:{'$regex': regex, '$options': 'i'}})
                        continue

                # for '==' and 'eq' conditions
                if op in ('==', 'eq'):
                    if len(value) == 1:
                        query.update({col:value[0]})
                    else:
                        query.update({col:{'$in':value}})
                    continue

                # for '!=' and 'neq' conditions
                if op in ('!=', 'neq'):
                    if len(value) == 1:
                        query.update({col:{'$ne': value[0]}})
                    else:
                        query.update({col:{'$not': {'$in':value}}})
                    continue

        logging.debug("DB query: ")
        logging.debug(query)
        return query

    def connect(self, db_name):
        self.conn = pymongo.Connection( host=self.MongoDBUri,
                                   _connect=False,
                                   read_preference=pymongo.ReadPreference.SECONDARY)
        self.db = self.conn[db_name]

    def insert(self, coll, data):
        return self.db[coll].insert(data)

    def update(self, coll, filter, data):
        query = self.__toDBFormat(filter)
        self.db[coll].update(query, {'$set': data}, upsert=True)

    def find(self, coll, filter, result={}):
        query = self.__toDBFormat(filter)
#        result = []
#        for item in self.db[coll].find(query):
#            item.pop('_id')
#            result.append(item)
#        return result
        if (result):
            return self.db[coll].find(query, result)
        else:
            return self.db[coll].find(query)

    def remove(self, coll, filter):
        query = self.__toDBFormat(filter)
        self.db[coll].remove(query)

#--------------------------------------------------------------
# The following is Solr implyment
# self.SolrDBUri = solr DB Uri
#--------------------------------------------------------------
class SolrDB(DB):
    def __init__(self):
        CLFCfg.init()
        self.SolrDBUri = CLFCfg.CLF_DB_SOLR_URI
        self.conn = None
        self.split = CLFCfg.CLF_Summary_SplitChar

    def __del__(self):
        if (not (self.conn is None)):
            self.conn.close()

    def __toDBFormat(self, filter):
        #maybe not support wildcard "*", only support range [xxx To xxx] and xxx:xxx
        query = ''
        ops = {'<': '[* TO %r}', '>': '{%r TO *]', '<=': '[* TO %r]', '>=': '[%r TO *]'}

        ESCAPE_CHARS_RE = re.compile(ur'(?<!\\)(?P<char>[&|+\-!(){}[\]^"~?:/ \u3000])')
        for subdict in filter:
            if subdict['op'] in ops:
                range  = ops[subdict['op']] % int(subdict['value'])
                #query.append(subquery)
                if len(query) == 0:
                    query = query + ''.join('%s:%s'%(subdict['col'],range))
                else:
                    query = query + ''.join(' AND %s:%s'%(subdict['col'],range))

            if subdict['op'] in ('neq', '!='):    #parse != or neq
                if not isinstance(subdict['value'],list):
                    subdict['value'] = [subdict['value']]
                for value in subdict['value']:
                    if subdict['op'] == 'neq':
                        query = query + ''.join(' NOT %s:%s'%(subdict['col'],value))
                    else:
                        query = query + ''.join(' NOT %s:%s'%(subdict['col'],value))


            if subdict['op'] == 'eq' or subdict['op'] == '==':
                if not isinstance(subdict['value'],list):
                    tmp_value =  subdict['value']
                    subdict['value'] = []
                    subdict['value'].append(tmp_value)
                if len(subdict['value']) > 1:
                    # should do "OR"
                    subquery = '('
                    for i, value in enumerate(subdict['value']):
                        if isinstance(value, str) or isinstance(value, unicode):
                            value = ESCAPE_CHARS_RE.sub(ur'\\\g<char>', value if isinstance(value, unicode) else unicode(value, 'utf-8'))
                        if len(subquery) == 1:
                            subquery = subquery + ''.join('%s:%s'%(subdict['col'],value))
                        else:
                            subquery = subquery + ''.join(' OR %s:%s'%(subdict['col'],value))
                    subquery = subquery +')'

                    if len(query) == 0:
                        query = query + ''.join('%s'% subquery)
                    else:
                        query = query + ''.join(' AND %s'%(subquery))

                else:
                    #for i, value in enumerate(subdict['value']):
                    value = subdict['value'][0]
                    if isinstance(value, str) or isinstance(value, unicode):
                        value = ESCAPE_CHARS_RE.sub(ur'\\\g<char>', value if isinstance(value, unicode) else unicode(value, 'utf-8'))
                    if len(query) == 0:
                        query = query + ''.join('%s:%s'%(subdict['col'],value))
                    else:
                        query = query + ''.join(' AND %s:%s'%(subdict['col'],value))

        return query



    def insert(self, coll, data):
        if not isinstance(data,list):
            data = [data]

        for data_ in data:
            assert(isinstance(data_,dict))

            if not data_.has_key('id'):
                data_['id'] = str(uuid.uuid1())

        self.conn.add_many(data)
        self.conn.commit()

    def update(self, coll, filter, data):
        try:
            query = self.__toDBFormat(filter)
        except Exception, e:
            logging.exception('update toDBFormat failed:%s', e)
            return -1 # return filter error
        logging.debug('update is %r',query)
        response = self.conn.query(query)
        content = response.results
        if len(content) > 0:
            self.conn.delete(query)
            self.conn.commit()
            data['id'] = content[0]['id']
        else:
            if not data.has_key('id'):
                data['id'] = str(uuid.uuid1())
        self.conn.add_many([data])
        self.conn.commit()
        return 0   # return OK



    def find(self, coll, filter, start=0, rows=10, sort='ts desc'):
        try:
            query = self.__toDBFormat(filter)
        except Exception, e:
            logging.exception('update  toDBFormat failed:%s', e)
            return None
        logging.debug('query is %r', query)
        #result = []
        response = self.conn.query(query,sort=sort,start=start,rows=rows)
        # numFound
        logging.debug('response.numFound : %r',(response.numFound))
        #result.extend(response.results)
        #start = response.start
        #num = len(response.results)
        #if num < response.numFound:
            ##query['start'] = num
            ##query['rows'] = (response.numFound - num)
            #response = self.conn.query(query,start=num,rows=response.numFound-num,sort='ts desc')
            #logging.debug('query second len is %r',len(response.results))
            #result.extend(response.results)
        #logging.debug('result len %r',len(result))
        #return result
        return response.results


    #The interface will parser filter for solr by facetName and summary_map

    def __parseFilter(self, facetName,filters):
        if not facetName:
            logging.error('parameters error!!!')
            return
        index_info = summary_map[facetName]
        if not index_info:
            logging.error('parameters error!!!')
            return
        index_name = index_info['index_name']
        index_value = index_info['value']
        if not filters:
            filters = []                  #if filter is null , return *
        ll = ['*']*len(index_value)
        for item in filters:
            if not item['col'] in index_value :
                continue

            col_index = index_value.index(item['col'])

            if isinstance(item['value'],list):
                if len(item['value']) == 1:   # may be we should enchance this solution to support muti-value
                    ll[col_index] = str(item['value'][0])
            else:
                ll[col_index] = str(item['value'])

        ESCAPE_CHARS_RE = re.compile(ur'(?<!\\)(?P<char>[&|+\-!(){}[\]^"~?:/ \u3000])')
        values =  self.split.join(ll)
        values = ESCAPE_CHARS_RE.sub(ur'\\\g<char>', values if isinstance(values, unicode) else unicode(values, 'utf-8'))
        return ''.join('%s:%s'%(index_name,values))


    #    filters = [{'col': CLF_UserName, 'op': 'eq', 'value': 'David Zhang (RD-CN)'},
    #          {'col': CLF_Domain, 'op': '=', 'value': 'www.baidu.com'},
    #          {'col': CLF_Device , 'op': '=', 'value': 'IWSVA6.0'}]

    def findEx(self, coll, companyID,facetName,filters, timeBegin, timeEnd,start=0, rows=10, sort = 'ts desc', scheduled_time_filter=None):
        if not companyID or not facetName or timeBegin < 0 or timeEnd < timeBegin:
            logging.error('findex parameter error:%r, %r, %r,%r',companyID,facetName,timeBegin,timeEnd)
            return None

        filter_cid = ''.join('%s:%s'%(CLFCfg.CLF_CompanyID,companyID))
        if not scheduled_time_filter or scheduled_time_filter['timeType'] == 0:
            filter_time = ''.join('%s:[ %r TO %r }'%(CLFCfg.CLF_TimeStamp,timeBegin, timeEnd))
        else:
            t_pair = clf_datetime.getTimePair(companyID,timeBegin, timeEnd,scheduled_time_filter, False)
            if not t_pair:
                return []
            filter_time = []
            for ts1,ts2 in t_pair:
                filter_time.append('%s:[ %r TO %r }' % (CLFCfg.CLF_TimeStamp, ts1, ts2))
            filter_time =  ' (%s) ' % (' OR '.join(filter_time))
        filter_value = self.__parseFilter(facetName,filters)
        ll = [filter_cid,filter_time,filter_value]
        query = ' AND '.join(ll)

        logging.debug('query:%r',query)
        response = self.conn.query(query,sort=sort,start=start,rows=rows)
        logging.debug('response.numFound : %r',(response.numFound))
        logging.debug('response.result : %r',(response.results))
        return response.results



    def remove(self, coll, filter):
        query = self.__toDBFormat(filter)
        logging.debug('remove is %r',query)
        self.conn.delete_query(query)
        self.conn.commit()

    def connect(self, db_name):
        self.conn =  solr.SolrConnection(self.SolrDBUri + '/' + db_name)


# --------------------------------------------------------------
# The following is Loacl disk storage stats data implyment
# storage dir is self.storage_dir
# storage mode:
#     ----cid
#            -----coll (dashboard, stats)
#                     ---- stats name
#                                    ----timestamp (day time as folder)
#                                                 -----time,interval,zip
#                                                 ----- 135478900_3600_0, 135478900_60_0, 135478900_900_0, and so on
#---------------------------------------------------------------
class LocalStorage(DB):
    '''LocalStorage implementation.'''
    def __init__(self):
        CLFCfg.init()
        #get storage folder:
        self.storage_dir = CLFCfg.CLF_STATS_LOCAL_DIK_DIR
        if not os.path.exists(self.storage_dir):
            os.makedirs(self.storage_dir)

    def __del__(self):
        #if (not (self.conn is None)):
           # self.conn.disconnect()
        pass

    def __toDBFormat(self, filter):
        para = collections.defaultdict()
        for item in filter:
            para[item['col']] = item['value']
        return para

    def connect(self, db_name):
        pass

    def insert(self, coll, data):
        return self.db[coll].insert(data)

    def update(self, coll, filter, data):
        para = self.__toDBFormat(filter)
        companyID =  para[CLFCfg.CLF_CompanyID]
        time = para[CLFCfg.CLF_TimeStamp]
        stat_name = para[CLFCfg.CLF_StatsName]
        interval = para[CLFCfg.CLF_StatsInterval]
        content = data[CLFCfg.CLF_Stats]
        zip = para[CLFCfg.CLF_StatsZip]
        #filename = cid_time_statName_interval
        filename = '%d_%d_%d'%(time,interval,zip)
        daytime =  time/(3600*24)*(3600*24)
        subdir = '%s/%s/%s/%d/'%(companyID,coll,stat_name,daytime)
        s = self.storage_dir +subdir+filename
        # should add file lock

        if os.path.exists(s):
            os.remove(s)
        if not os.path.exists(self.storage_dir+subdir):
            os.makedirs(self.storage_dir+subdir)
        stats_file = open(s,'wb+')
        #content
        stats_file.write(content)
        stats_file.close()
        #stats_file.close()
        #query = self.__toDBFormat(filter)
        #self.db[coll].update(query, {'$set': data}, upsert=True)
    '''
        return self.db.find(CLFCfg.CLF_DB_STATS, [
                        {'col': CLFCfg.CLF_CompanyID, 'op': 'eq', 'value': companyID},
                        {'col': CLFCfg.CLF_StatsName, 'op': 'eq', 'value': stat_name},
                        {'col': CLFCfg.CLF_TimeStamp, 'op': '>=', 'value': timeBegin},
                        {'col': CLFCfg.CLF_TimeStamp, 'op': '<', 'value': timeEnd}])
    '''
    def find(self, coll, filter):
        #query = self.__toDBFormat(filter)
        # parameter parse
        # CLFCfg.CLF_DB_DASHBOARD_STATS
        cid = None
        stat_name = None
        timeBegin = None
        timeEnd = None
        interval = None
        result = []
        for para in filter:
            if para['col'] == CLFCfg.CLF_CompanyID:
                cid = para['value']
                continue
            if para['col'] == CLFCfg.CLF_StatsName:
                stat_name = para['value']
                continue
            if para['col'] == CLFCfg.CLF_TimeStamp:
                if para['op'] == '>=':
                    timeBegin = para['value']
                    continue
                if para['op'] == '<':
                    timeEnd = para['value']
                    continue
            if para['col']  == CLFCfg.CLF_StatsInterval:
                interval = para['value']

        if cid is not None and  stat_name is not None and timeBegin is not None and  timeEnd is not None:
            delPath = self.storage_dir
            time_range = timeEnd-timeBegin
            rangeNum =  time_range/(24*3600)
            daytime_begin = timeBegin/(24*3600)*(24*3600)
            for i in xrange(rangeNum+1):
                daytime = daytime_begin + i*24*3600 #daytime as folder name
                subdir = '%s/%s/%s/%d/'%(cid,coll,stat_name,daytime)
                if os.path.exists(delPath+subdir):
                    filenames=os.listdir(delPath+subdir)
                    for filename  in filenames:
                        #filename = time,interval,zip
                        m = re.match('^(\d+)_(\d+)_(\d+)$',filename)
                        timestamp = int(m.group(1))
                        if interval and interval != int(m.group(2)):
                            continue
                        if timestamp >= timeBegin and timestamp < timeEnd :
                            #should get this result and put result to list
                            f = open(delPath+subdir+filename,'r')
                            content = f.read()
                            contentdict = dict()
                            contentdict['cid'] = cid
                            contentdict['dur'] = int(m.group(2))
                            contentdict['ts'] = timestamp
                            contentdict['zip'] = int(m.group(3))
                            contentdict['stats'] = content
                            f.close()
                            result.append(contentdict)
        else:
            logging.error('parameters have some error cid :%r, stats:%r, timeB:%r, timeEnd:%s',cid,stat_name,timeBegin,timeEnd)

        return result
        #return [x for x in self.db[coll].find(query)]

    def remove(self, coll, filter):
        pass
       # query = self.__toDBFormat(filter)
        #self.db[coll].remove(query)
class Sqlite(DB):

    MERGE_TIME  = 'Day'  #'WeeK', 'Month',and other
    MERGE_COUNT = 24 # if merge_time is day , this value is 24, if merge time is week, this value is 7, if merge time month, this value is 31
    S1= 'h0,h1,h2,h3,h4,h5,h6'
    S2= 'h0,h1,h2,h3,h4,h5,h6,h7,h8,h9,h10,h11,h12,h13,h14,h15,h16,h17,h18,h19,h20,h21,h22,h23'
    S3= 'h1,h2,h3,h4,h5,h6,h7,h8,h9,h10,h11,h12,h13,h14,h15,h16,h17,h18,h19,h20,h21,h22,h23,h24,h25,h26,h27,h28,h29,h30,h31'
    REPACE_SQL1 = 'replace into stats values (?,?,?,?,?,?,?'
    REPACE_SQL2 = 'replace into stats values (?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?'
    REPACE_SQL3 = 'replace into stats values (?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?'
    FACET_REPACE_SQL1= 'replace into facet values (?,?,?,?,?,?,?'
    FACET_REPACE_SQL2 = 'replace into facet values (?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?'
    FACET_REPACE_SQL3 = 'replace into facet values (?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?'
    STATS2_REPACE_SQL1= 'replace into stats_2 values (?,?,?,?,?,?,?'
    STATS2_REPACE_SQL2 = 'replace into stats_2 values (?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?'
    STATS2_REPACE_SQL3 = 'replace into stats_2 values (?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?'
    COUNT_TABLE1= 'h0 INTEGER, h1 INTEGER,h2 INTEGER,h3 INTEGER,h4 INTEGER,h5 INTEGER,h6 INTEGER'
    COUNT_TABLE2 = '''h0 INTEGER, h1 INTEGER,h2 INTEGER,h3 INTEGER,h4 INTEGER,h5 INTEGER,h6 INTEGER,h7 INTEGER,
                   h8 INTEGER, h9 INTEGER,h10 INTEGER,h11 INTEGER,h12 INTEGER,h13 INTEGER,h14 INTEGER,h15 INTEGER,
                   h16 INTEGER, h17 INTEGER,h18 INTEGER,h19 INTEGER,h20 INTEGER,h21 INTEGER,h22 INTEGER,h23 INTEGER
                '''
    COUNT_TABLE3 = ''' h1 INTEGER,h2 INTEGER,h3 INTEGER,h4 INTEGER,h5 INTEGER,h6 INTEGER,h7 INTEGER,
                   h8 INTEGER, h9 INTEGER,h10 INTEGER,h11 INTEGER,h12 INTEGER,h13 INTEGER,h14 INTEGER,h15 INTEGER,
                   h16 INTEGER, h17 INTEGER,h18 INTEGER,h19 INTEGER,h20 INTEGER,h21 INTEGER,h22 INTEGER,h23 INTEGER,
                   h24 INTEGER, h25 INTEGER,h26 INTEGER,h27 INTEGER,h28 INTEGER,h29 INTEGER,h30 INTEGER,h31 INTEGER
                '''

    def __init__(self, merge_month=False):
        CLFCfg.init()
        self.memoryDB = {}    # this dict will store memoryDB
        self.dashboardDB = {}
        #CLFCfg.CLF_DB_MONGODB_URI
        if merge_month:
            self.merge_mode = 'month'
        else:
            self.merge_mode = CLFCfg.CLF_STATS_MERGE_MODE # the parameter is configrable in configurantion file, 'day', 'week', 'month'

        self.stats_db_path = CLFCfg.CLF_STATS_STORE_PATH+'stats/'
        self.dashboard_path = CLFCfg.CLF_STATS_STORE_PATH+'dashboard/'
        if not os.path.exists(self.stats_db_path):
            os.makedirs(self.stats_db_path)
        if not os.path.exists(self.dashboard_path):
            os.makedirs(self.dashboard_path)
        self.table = {}
        #self.summary_table = None
        for stats_name, table_list in reportStats.iteritems():
           sql,filter,facet,facet_list,month_sql,month_facet,stats2,month_stats2 = self.__parseTable(stats_name,table_list['logFields'])
           stats_table = {'stats':sql,'month_stats':month_sql,'filter':filter,'facet':facet,'month_facet':month_facet,'facet_list':facet_list,'stats2':stats2,'month_stats2':month_stats2} # fiter xxx = ? and xxx = ? .....
           self.table[stats_name] = stats_table # create table sql: create table stats (xxx TEXT, xxx TEXT,...,h0 INTEGER, ..., h23 INTEGER)

        # shuld create db file to manage all database
        # table : cid, stats_name, timestamp
        db_name = ''.join('%smanage_db_file.db'%self.stats_db_path)
        self.manage_db = sqlite3.connect(db_name, check_same_thread = False, timeout = 10)
        self.manage_cur =  self.manage_db.cursor()

        # manage stats day table
        self.manage_cur.execute('CREATE TABLE IF NOT EXISTS statsname (cid TEXT, stats_name TEXT, ts INTEGER,db_name TEXT,PRIMARY KEY(cid,stats_name,ts))')

        # manage stats month table
        self.manage_cur.execute('CREATE TABLE IF NOT EXISTS month_statsname (cid TEXT, stats_name TEXT, ts INTEGER,db_name TEXT,PRIMARY KEY(cid,stats_name,ts))')

        #manage stats for calculating month table from day table
        self.manage_cur.execute('CREATE TABLE IF NOT EXISTS time (day_time INTEGER, day_hour INTEGER, month_time INTEGER, time_zone TEXT)')

        #monthtable lock database
        self.month_lock_dbfile =  self.stats_db_path + 'month_lock.db'
        #self.month_lock_db = sqlite3.connect(month_lock_dbfile,check_same_thread = False)
        #self.month_lock_cur = self.month_lock_db.cursor()
        #self.month_lock_cur.execute('CREATE TABLE IF NOT EXISTS lock (month_lock_status INTEGER)')

        self.browse_time_db = None
        self.__getCentralDeviceTimeZone()

            #self.table.append(s)
    def __getMergeSql(self,merge_count):
        if not isinstance(merge_count,int):
            logging.info('parameter have error')
            return ''
        s = ''
        for i in xrange(merge_count):
            tmp_s = 'h%d INTEGER'%i
            s += ','
            s += tmp_s
        return s
    def __getCentralDeviceTimeZone(self):  # get central device Time zone
        # should read timezone from Central Device
        self.zone = pytz.utc #[Asia/Shanghai, Asia/Harbin, Asia/Chongqing, Asia/Urumqi, Asia/Kashgar]
        return self.zone

    def __getTimezoneRange(self, timestamp):
        utc_time = time.mktime(datetime.utcfromtimestamp(timestamp).timetuple())
        loc_time = time.mktime(datetime.fromtimestamp(timestamp,self.zone).timetuple())
        return  (loc_time-utc_time)/3600

    def __translateUTCtoTimezone(self, cid, ts):
        date_zone = datetime.datetime.fromtimestamp(ts, self.zone)
        t3 = calendar.timegm(date_zone.timetuple())
        return t3

    def __parseTable(self, tablename, table_list):
        s  = 'CREATE TABLE IF NOT EXISTS stats ('
        #create table for the first facet page
        # facet ()
        facet = 'CREATE TABLE IF NOT EXISTS facet (columnID TEXT,columnIDType TEXT,CLF_DeviceName TEXT, '
        #create table for mime_type and category
        # CLF_DeviceName, CLF_URLCategory, CLF_CustomURLCategory, CLF_ClientIP, CLF_ServerIP,
        # CLF_UserName, CLF_Domain, count1, count2
        stats_level2 = 'CREATE TABLE IF NOT EXISTS stats_2(CLF_DeviceName varchar(64),CLF_URLCategory smallint ,CLF_CustomURLCategory varchar(64) ,columnID varchar(300),columnIDType varchar(20), '

        filter = ''
        primary_key = 'PRIMARY KEY ('
        for item in table_list:
            s = s + ''.join('%s TEXT, '%item)

        primary_key += ','.join(table_list)
        primary_key += ')'

        filter += '=? AND '.join(table_list)
        filter += '=?'
        #if self.merge_mode == 'day':
        s_day =  s + self.COUNT_TABLE2  # config
        facet_day = facet + self.COUNT_TABLE2
        stats_level2_day= stats_level2 + self.COUNT_TABLE2
        #elif self.merge_mode == 'month':
        s_month = s + self.COUNT_TABLE3
        facet_month = facet + self.COUNT_TABLE3
        stats_level2_month= stats_level2 + self.COUNT_TABLE3

        s_day += ','
        s_day += primary_key
        s_day = s_day+')'

        s_month += ','
        s_month += primary_key
        s_month += ')'

        facet_day += ', PRIMARY KEY (columnID,columnIDType,CLF_DeviceName))'
        facet_month += ', PRIMARY KEY (columnID,columnIDType,CLF_DeviceName))'
        stats_level2_day += ', PRIMARY KEY (CLF_DeviceName,CLF_URLCategory,CLF_CustomURLCategory,columnID,columnIDType))'
        stats_level2_month += ', PRIMARY KEY (CLF_DeviceName,CLF_URLCategory,CLF_CustomURLCategory,columnID,columnIDType))'

        columIDType_list = table_list[:]

        return s_day, filter,facet_day,columIDType_list,s_month,facet_month,stats_level2_day,stats_level2_month

    def __del__(self):
        for db in self.memoryDB.itervalues():
            db.close()
        for dash_db in self.dashboardDB.itervalues():
            dash_db.close()
        if self.manage_db is not None:
            self.manage_db.close()

    def __toDBFormat(self, filter):
        para = collections.defaultdict()
        for item in filter:
            para[item['col']] = item['value']
        return para

    def insert(self, coll, companyID, time, stat_name, interval, content, arg = None):
        logging.debug('please use update to insert')
        pass

    # timestamp is UTC , return : hour is Timezone local hour, dayTime is start time (0:00:0) of UTC
    def __getTimestampStartDayTime(self, cid,timestamp,timezone=None):
        # first translate timestamp into Timezone's datetime
        timestamp = (timestamp/3600)*3600
        if timezone:
            date_zone = datetime.datetime.fromtimestamp(timestamp, timezone)
        else:
            date_zone = datetime.datetime.fromtimestamp(timestamp, self.zone)
        hour = date_zone.hour # get hour of timezone
        #tmp_date = datetime.datetime(date_zone.year,date_zone.month,date_zone.day,0,0)
        tmp_date = date_zone.replace(hour=0)

        dayTime = calendar.timegm(tmp_date.utctimetuple())
        return hour, dayTime
        
    def __getTimestampStartHourUTCtimestamp(self, cid, UTC_begin,h):
        date = datetime.datetime.fromtimestamp(UTC_begin,self.zone)
        tmp_date = date.replace(hour=h)
        timestamp = calendar.timegm(tmp_date.utctimetuple())
        return timestamp

    def __getTimestampStartDayUTCtimestamp(self, cid, UTC_begin,d):
        timezone,_ = self.__get_localzone()
        date = datetime.datetime.fromtimestamp(UTC_begin,timezone)
        #else:
           # date = datetime.datetime.fromtimestamp(UTC_begin,self.zone)

        tmp_date = date.replace(day=d)
        timestamp = calendar.timegm(tmp_date.utctimetuple())
        return timestamp

    # timestamp is UTC , return : day is Timezone local day(1), dayTime is start time (xxxx/xx/1/0:00:0) of UTC
    def __getTimestampStartMonthTime(self, cid, timestamp, timezone = None):
        # first translate timestamp into Timezone's datetime
        timestamp = (timestamp/3600)*3600
        if timezone:
            date_zone = datetime.datetime.fromtimestamp(timestamp, timezone)
        else:
            date_zone = datetime.datetime.fromtimestamp(timestamp, self.zone)

        day = date_zone.day # get hour of timezone
        #tmp_date = date_zone.replace(hour=0)
        #tmp_date = datetime.datetime(date_zone.year,date_zone.month,1,0,0)
        tmp_date = date_zone.replace(day=1,hour=0)
        monthTime = calendar.timegm(tmp_date.utctimetuple())
        return day, monthTime

    def __getMonthRange(self, cid, timestamp, timezone = None):
        if timezone:
            date_zone = datetime.datetime.fromtimestamp(timestamp, timezone)
        else:
            date_zone = datetime.datetime.fromtimestamp(timestamp, self.zone)
        return calendar.monthrange(date_zone.year, date_zone.month)[1]

    def __createIndexForDB(self, stats_name, db, cur):
         # for stats_name db create index
         if not stats_name or not db or not cur:
             logging.error('create index for db para error!')
             return
         if not facet_index.has_key(stats_name):
             return
         else:
             index_list =  facet_index[stats_name]
         for index_item in index_list:
             #index_item = 'CLF_xxx'
             sql_l = ''.join('create index if not exists %s_index on stats(%s)'%(index_item,index_item))
             cur.execute(sql_l)
         db.commit()


    def createdb(self, timestamp, stats_name,cid, month_db = False):
        if month_db:
            db_name = ''.join('%s%s_%s_%r_month.db'%(self.stats_db_path,cid,stats_name,timestamp))
        else:
            db_name = ''.join('%s%s_%s_%r.db'%(self.stats_db_path,cid,stats_name,timestamp))
        try:
            if not self.memoryDB.has_key(db_name):
                db = sqlite3.connect(db_name)
                cur =  db.cursor()
                page_size = cur.execute('PRAGMA page_size').fetchone()[0]
                if page_size != 32768: #32K
                    cur.execute('PRAGMA page_size = 32768;')
                    db.commit()
                    cur.execute('VACUUM')
                    db.commit()

                cur.execute(self.table[stats_name]['stats'])
                # for facet table
                cur.execute(self.table[stats_name]['facet'])
                self.__createIndexForDB(stats_name, db, cur)
                if stats_name == 'facet_internet_access':
                    cur.execute(self.table[stats_name]['stats2'])

                self.manage_cur.execute('REPLACE INTO statsname VALUES(?,?,?,?)', [cid,stats_name,timestamp,db_name])
                self.manage_db.commit()
                old_db=''.join('%s%s_%s'%(self.stats_db_path,cid,stats_name))
                keys  = self.memoryDB.keys()
                for k in keys:
                    m = re.match(old_db, k)
                    if m is not None:
                        old_db_name = self.memoryDB.pop(k)
                        logging.debug('old_db:%r, old_db_name:%r',old_db, old_db_name)
                        old_db_name.close()

                self.memoryDB[db_name] = db
        except Exception, e:
            logging.exception('create memoryDB fail %r',e)
        return self.memoryDB[db_name]

    def calculate_conter(self, t_index, value,old_counter):
        if old_counter :
            ll = list(old_counter)
        else:
            if self.merge_mode == 'day':
                ll = [0]*24   # config ll = [0]*31
            elif self.merge_mode == 'month':
                ll = [0]*31
            else:
                ll = [0]*7
        if self.merge_mode == 'month':
            if isinstance(t_index,list):
                ll[t_index[0]-1] += int(value[0])
                ll[t_index[1]-1] += int(value[1])
            else:
                ll[t_index-1] += int(value)
        else:
            ll[t_index] += int(value)

        return ll


    # Counter({('', 'sales rep1'): 3, ('', 'sales rep6'): 2, ('', 'sales rep9'): 1, ('', 'sales rep7'): 1,
    # ('', 'sales rep4'): 1, ('', 'sales rep10'): 1, ('', 'sales rep11'): 1}) collections.Counter()
    def db_update(self, content, hour, db,stats_name,cid):
        t_hour = hour
        if self.merge_mode == 'day':
            s2 = self.S2
            sql2 = self.REPACE_SQL2
            facet_repace_sql =self.FACET_REPACE_SQL2
            stats2_repace_sql = self.STATS2_REPACE_SQL2
        elif self.merge_mode == 'month':
            s2 = self.S3
            sql2 = self.REPACE_SQL3
            facet_repace_sql =self.FACET_REPACE_SQL3
            stats2_repace_sql = self.STATS2_REPACE_SQL3
        else:
            s2 = self.S1
            sql2 = self.REPACE_SQL1
            facet_repace_sql =self.FACET_REPACE_SQL1
            stats2_repace_sql = self.STATS2_REPACE_SQL3

        filter = self.table[stats_name]['filter']
        facet_list = self.table[stats_name]['facet_list']

        sql = ''.join('SELECT %s FROM stats WHERE %s'%(s2, filter)) #CLF_Devicename
        facet_sql = ''.join('SELECT %s FROM facet WHERE columnID = ? and columnIDType= ? and CLF_DeviceName=? '%s2)
        stats2_sql = ''.join('SELECT %s FROM stats_2 WHERE CLF_DeviceName = ?  and CLF_URLCategory=? and CLF_CustomURLCategory=? and columnID=? and columnIDType=?'%s2)
        logging.debug('db_update first query sql:%r',sql)
        insert_list = []
        facet_repace_sql +=',?,?,?)'
        stats2_repace_sql += ',?,?,?,?,?)'
        try:
            stats_sqlit_data = []
            facet_sqlit_data = []
            stats2_sqlit_data = []
            facet_tmp_dict = {}
            stats2_dict = {}
            for k, v in content.iteritems():
                c = db.cursor()
                result = c.execute(sql,list(k)).fetchone()  # k = (xx,xx,)
                new_counter = self.calculate_conter(t_hour,v,result)
                repace_sql = sql2 + ',?'*len(k) + ')'
                logging.debug('repace_sql:%r',repace_sql)
                logging.debug('new counter:%r',new_counter)
                #c.execute(repace_sql,list(k)+new_counter)
                stats_sqlit_data.append(list(k)+new_counter)
                #update facet table CLF_DeviceName and 'CLF_DeviceName' in k
                if len(k) >=4 and  'CLF_DeviceName' in reportStats[stats_name]['logFields']:
                    deviceID =  reportStats[stats_name]['logFields'].index('CLF_DeviceName')
                    columnIds = list(k)
                    for i, columnId in enumerate(columnIds):
                        #facet_sql = ''.join('SELECT %s FROM stats WHERE %s'%(s2, filter))
                        if facet_tmp_dict.has_key((columnId,facet_list[i],columnIds[deviceID])):
                            tmp_reslut = facet_tmp_dict[(columnId,facet_list[i],columnIds[deviceID])]
                            facet_tmp_dict[(columnId,facet_list[i],columnIds[deviceID])] = self.calculate_conter(t_hour,v,tmp_reslut)
                        else:
                            result = c.execute(facet_sql,[columnId,facet_list[i],columnIds[deviceID]]).fetchone()  # k = (xx,xx,)
                            new_facet_counter = self.calculate_conter(t_hour,v,result)
                            facet_tmp_dict[(columnId,facet_list[i],columnIds[deviceID])] = new_facet_counter
                        #logging.debug('repace_sql:%r',repace_sql)
                        #logging.debug('new counter:%r',new_facet_counter)
                        #c.execute(facet_repace_sql,[columnId,facet_list[i],columnIds[deviceID]]+new_facet_counter)
                        #facet_sqlit_data.append([columnId,facet_list[i],columnIds[deviceID]]+new_facet_counter)
                if stats_name == 'facet_internet_access':
                #CLF_DeviceName TEXT,CLF_URLCategory TEXT ,
                # CLF_CustomURLCategory TEXT ,columnID TEXT,columnIDType TEXT
                # 'logFields': ['CLF_DeviceName', 'CLF_UserName',  'CLF_ClientIP',  'CLF_Domain', 'CLF_URLCategory', 'CLF_CustomURLCategory'],
                    if stats2_dict.has_key((k[0],k[4],k[5],k[1],'CLF_UserName')):
                        tmp_reslut2 = stats2_dict[(k[0],k[4],k[5],k[1],'CLF_UserName')]
                        stats2_dict[(k[0],k[4],k[5],k[1],'CLF_UserName')] = self.calculate_conter(t_hour,v,tmp_reslut2)
                    else:
                        result = c.execute(stats2_sql,[k[0],k[4],k[5],k[1],'CLF_UserName']).fetchone()
                        new_stats2_counter = self.calculate_conter(t_hour,v,result)
                        stats2_dict[(k[0],k[4],k[5],k[1],'CLF_UserName')] = new_stats2_counter

                    if stats2_dict.has_key((k[0],k[4],k[5],k[2],'CLF_ClientIP')):
                        tmp_reslut2 = stats2_dict[(k[0],k[4],k[5],k[2],'CLF_ClientIP')]
                        stats2_dict[(k[0],k[4],k[5],k[2],'CLF_ClientIP')] = self.calculate_conter(t_hour,v,tmp_reslut2)
                    else:
                        result = c.execute(stats2_sql,[k[0],k[4],k[5],k[2],'CLF_ClientIP']).fetchone()
                        new_stats2_counter = self.calculate_conter(t_hour,v,result)
                        stats2_dict[(k[0],k[4],k[5],k[2],'CLF_ClientIP')] = new_stats2_counter


                    if stats2_dict.has_key((k[0],k[4],k[5],k[3],'CLF_Domain')):
                        tmp_reslut2 = stats2_dict[(k[0],k[4],k[5],k[3],'CLF_Domain')]
                        stats2_dict[(k[0],k[4],k[5],k[3],'CLF_Domain')] = self.calculate_conter(t_hour,v,tmp_reslut2)
                    else:
                        result = c.execute(stats2_sql,[k[0],k[4],k[5],k[3],'CLF_Domain']).fetchone()
                        new_stats2_counter = self.calculate_conter(t_hour,v,result)
                        stats2_dict[(k[0],k[4],k[5],k[3],'CLF_Domain')] = new_stats2_counter

                        #logging.debug('repace_sql:%r',repace_sql)
                        #logging.debug('new counter:%r',new_facet_counter)
                        #c.execute(facet_repace_sql,[columnId,facet_list[i],columnIds[deviceID]]+new_facet_counter)
                        #facet_sqlit_data.append([columnId,facet_list[i],columnIds[deviceID]]+new_facet_counter)
            for facet_key, values in facet_tmp_dict.iteritems():
                facet_sqlit_data.append(list(facet_key)+values)
            for stats2_key, stats_values in stats2_dict.iteritems():
                stats2_sqlit_data.append(list(stats2_key)+stats_values)
            if stats_sqlit_data:
                c.executemany(repace_sql,stats_sqlit_data)
            if facet_sqlit_data:
                c.executemany(facet_repace_sql,facet_sqlit_data)
            if stats2_sqlit_data:
                c.executemany(stats2_repace_sql,stats2_sqlit_data)
        except Exception, e:
            logging.exception('update data to DB fail %r',e)
        #db.commit()

     # self, coll, companyID, time, stat_name, interval, content, arg = None
    def update(self, coll, filter, data):
        #c.execute('UPDATE stocks SET price=? WHERE date = ?',(50, '2006-04-05'))
        para = self.__toDBFormat(filter)
        companyID =  para[CLFCfg.CLF_CompanyID]
        time = para[CLFCfg.CLF_TimeStamp]
        stat_name = para[CLFCfg.CLF_StatsName]
        interval = para[CLFCfg.CLF_StatsInterval]
        content = data[CLFCfg.CLF_Stats]
        if coll == CLFCfg.CLF_DB_DASHBOARD_STATS:
            self.update_dashboard(companyID,time,stat_name,interval,content)

        if coll == CLFCfg.CLF_DB_STATS:
            if interval == 300:
                logging.debug('this is browse time data')
                self.update_browse_time_stats(companyID,time,stat_name,interval,content)
            else:
                if self.merge_mode == 'day':
                    hour, time_start_timestamp = self.__getTimestampStartDayTime(companyID,time)
                if self.merge_mode == 'month':
                    hour, time_start_timestamp = self.__getTimestampStartMonthTime(companyID,time)

                db = self.createdb(time_start_timestamp, stat_name,companyID)
                self.db_update(content, hour, db,stat_name,companyID)
                db.commit()

    def update_browse_time_stats(self,cid, ts, stats_name,dur, content):
        db_name = self.stats_db_path + 'browse_time.db'
        try:
            if not self.browse_time_db:
                db = sqlite3.connect(db_name)
                cur =  db.cursor()
                sql = ''.join('CREATE TABLE IF NOT EXISTS stats (%s TEXT, %s INTEGER,%s TEXT,%s INTEGER,%s TEXT,PRIMARY KEY (%s,%s,%s,%s))'%(CLFCfg.CLF_CompanyID,CLFCfg.CLF_TimeStamp,CLFCfg.CLF_StatsName,CLFCfg.CLF_StatsInterval, CLFCfg.CLF_Stats,CLFCfg.CLF_CompanyID,CLFCfg.CLF_TimeStamp,CLFCfg.CLF_StatsName,CLFCfg.CLF_StatsInterval))
                sql_ts = ''.join('CREATE TABLE IF NOT EXISTS timestamp (%s TEXT,%s TEXT, %s INTEGER,PRIMARY KEY(%s,%s))'%(CLFCfg.CLF_CompanyID,CLFCfg.CLF_StatsName,CLFCfg.CLF_TimeStamp,CLFCfg.CLF_CompanyID,CLFCfg.CLF_StatsName))
                #create table timestamp (cid, stats_name, ts , primary key (cid, stats_name))
                #cur.execute('CREATE TABLE IF NOT EXISTS stats (cid TEXT, ts INTEGER,statsname TEXT,dur INTEGER,doc TEXT,PRIMARY KEY (cid,ts,statsname,dur))')
                cur.execute(sql)  # create table to store merge 300s data to hour data flag
                cur.execute(sql_ts) # create table to store 300s stats data
                if not content:
                    cur.execute('replace into timestamp values (?,?,?)',[cid,stats_name,ts])
                    db.commit()
                else:
                    cur.execute('replace into stats values (?,?,?,?,?)',[cid,ts,stats_name,dur,content])
                    db.commit()
                self.browse_time_db = db
            else:
                db = self.browse_time_db
                cur =  db.cursor()
                if not content:
                    cur.execute('replace into timestamp values (?,?,?)',[cid,stats_name,ts])
                else:
                    cur.execute('replace into stats values (?,?,?,?,?)',[cid,ts,stats_name,dur,content])
                db.commit()

        except Exception, e:
            logging.exception('update browse time stats data to DB fail %r',e)
            return -1

    def update_dashboard(self,cid, ts, stats_name,dur, content):
        db_name = ''.join('%s%s_%s.db'%(self.dashboard_path,cid,stats_name))
        try:
            if not self.dashboardDB.has_key(db_name):
                db = sqlite3.connect(db_name)
                cur =  db.cursor()
                sql = ''.join('CREATE TABLE IF NOT EXISTS dashboard (%s TEXT,%s INTEGER,%s TEXT, %s INTEGER,stats TEXT,PRIMARY KEY(%s,%s,%s,%s) )'%(CLFCfg.CLF_CompanyID,CLFCfg.CLF_TimeStamp,CLFCfg.CLF_StatsName,CLFCfg.CLF_StatsInterval,CLFCfg.CLF_CompanyID,CLFCfg.CLF_TimeStamp,CLFCfg.CLF_StatsName,CLFCfg.CLF_StatsInterval))
                cur.execute(sql)
                #cur.execute('CREATE TABLE IF NOT EXISTS dashboard (cid TEXT, ts INTEGER,statsname TEXT,dur INTEGER,doc TEXT,PRIMARY KEY (cid,ts,statsname,dur))')
                cur.execute('replace into dashboard values (?,?,?,?,?)',[cid,ts,stats_name,dur,content])
                db.commit()
                self.dashboardDB[db_name] = db
            else:
                db = self.dashboardDB[db_name]
                cur =  db.cursor()
                cur.execute('replace into dashboard values (?,?,?,?,?)',[cid,ts,stats_name,dur,content])
                db.commit()

        except Exception, e:
            logging.exception('update dashboard data to DB fail %r',e)
            return -1

    def __query_filter_parse(self,filter):
        filter_list = []               #
        columnID_list = []
        filter_sql = ''
        if filter is not None:
            for columnID, filter_value in filter.iteritems():
                tmp_filter = []   # xxx:xxx or xxx:xxx
                for value in filter_value:
                    filter_list.append(value)
                   # columnID_list.append(columnID)
                    tmp_filter.append(columnID)
                if len(tmp_filter) > 1:
                    filter_subsql = '('
                    filter_subsql += '=? or '.join(tmp_filter)
                    filter_subsql += '=? )'
                    columnID_list.append(filter_subsql)
                else:
                    columnID_list.append(tmp_filter[0]+'=?')
        filter_sql = ' and '.join(columnID_list)
        return filter_sql, filter_list     # filter_sql: CLF_XXX = ? and (CLF_XXX=? or CLF_XXX=?) and CLF_XXX=?, filter_list: ['xxx', 'xxx','xxx','xxx']

    def __query_sum_time_parse(self, start_end_list):
       ll = []
       ll_ladp=[]
       for start_time,end_time in start_end_list:
           for i in range(start_time, end_time+1):
               tmp = ''.join('sum(h%d)'%i)
               tmp_ldap = ''.join('h%d'%i)
               ll.append(tmp)
               ll_ladp.append(tmp_ldap)
       return ll, ll_ladp # ll: sum(h0), sum(h1), sum(h2), .... ll_ladp: h1, h2, h3, h4, ......


    def __query_topOrMulti_top(self,db,cursor,top_name,filter,is_ldap_filter, start_end_list, stats_name):
        result = []
        top_result = {}
        sum_sql, sum_ldap = self.__query_sum_time_parse(start_end_list)
        filter_sql, filter_list = self.__query_filter_parse(filter)
        if filter and  len (filter) > 0:
            if is_ldap_filter: # ldap_filter is True, ignore top_name parameters stat_name
                sql = 'select '+ ','.join(reportStats[stats_name]['logFields'])+', '
                sql += '+'.join(sum_ldap)
                sql += ' from stats where '
                sql += filter_sql
                #sql = 'select  CLF_XXX, CLF_XX,CLF_XXX , h0+h1+..+hr from stats where filter'
                logging.debug('query with filter and ldap filter %s',sql)
                try:
                    ret = cursor.execute(sql,filter_list).fetchall() # select * from table where xx=? and xx = ?. result = [(xx,xx,h1,...,hn),(xx,xx,h1,...)]
                    result.extend(ret)          # [(CLF_XXX,CLF_XXX,CLF_XXX, 0),(CLF_XXX,CLF_XXX,CLF_XXX, 1),(CLF_XXX,CLF_XXX,CLF_XXX, 2),....]
                except Exception,e:
                    logging.exception('query error %r',e)
            else: #is_ldap_filter is None
                 # query facet data , filter is only device = 'xxx'
                facet_query = len(reportStats[stats_name]['logFields']) >= 4 and len(filter) == 1 and 'CLF_DeviceName' in filter
                sql = 'select '
                #check if  CLF_URLCategory or CLF_CustomURLCategory
                is_columnID =  'CLF_URLCategory' in filter or 'CLF_CustomURLCategory' in filter
                stats2_query = (stats_name == 'facet_internet_access' and is_columnID and (len(filter) == 1 or (len(filter) == 2 and 'CLF_DeviceName' in filter)))
                for top_column in top_name:
                    if facet_query:  # if query facet data, and filter is device name = 'xxx', query data in facet table
                        sql += 'columnID, '
                        sql += '+'.join(sum_ldap)
                        sql += ' from facet '
                        sql += 'where columnIDType=? and CLF_DeviceName=? '
                        try:
                            logging.debug('sql:%r',sql)
                            ret = cursor.execute(sql,[top_column,filter['CLF_DeviceName'][0]]).fetchall()
                            top_result[top_column] = ret
                        except Exception,e:
                            logging.exception('have no any filter query error %r',e)
                    else:
                        if stats2_query:
                            #'CLF_DeviceName', 'CLF_UserName', 'CLF_MimeType', 'CLF_ClientIP', 'CLF_ServerIP', 'CLF_Domain', 'CLF_URLCategory', 'CLF_CustomURLCategor
                            if top_column == 'CLF_UserName' or top_column == 'CLF_ClientIP' or top_column == 'CLF_ServerIP' or top_column == 'CLF_Domain':
                                sql += 'columnID,'
                                sql += '+'.join(sum_sql)
                                sql += ' from stats_2 where '
                                sql += filter_sql
                                sql += ' and columnIDType=?'
                                sql += ''.join(' group by columnID')
                                try :
                                    #sql: select CLF_XXX, sum(h0)+sum(h1)+..+sum(hr) from stats where filter group by CLF_XXX
                                    logging.debug('sql:%r and filter :%r',sql,filter_list)
                                    ret = cursor.execute(sql, filter_list+[top_column]).fetchall() # [('xxx',10),(xx,90),...]
                                    top_result[top_column] = ret
                                # result.append(top_result) # [{'CLF_XXX':[('xx',10),('xx',20)....]},{'CLF_XXX':[('xxx':10),..]},...]
                                except Exception,e:
                                    logging.exception('top query error %r',e)
                            else:
                                sql += ''.join('%s,'%top_column)
                                #sql += 'columnID,'
                                sql += '+'.join(sum_sql)
                                sql += ' from stats_2 where '
                                sql += filter_sql
                                sql += ' and columnIDType=?'
                                sql += ''.join(' group by %s'%top_column)
                                try :
                                    #sql: select CLF_XXX, sum(h0)+sum(h1)+..+sum(hr) from stats where filter group by CLF_XXX
                                    logging.debug('sql:%r and filter :%r',sql,filter_list)
                                    ret = cursor.execute(sql, filter_list+['CLF_ClientIP']).fetchall() # [('xxx',10),(xx,90),...]
                                    top_result[top_column] = ret
                                # result.append(top_result) # [{'CLF_XXX':[('xx',10),('xx',20)....]},{'CLF_XXX':[('xxx':10),..]},...]
                                except Exception,e:
                                    logging.exception('top query error %r',e)

                        else:
                            sql += ''.join('%s,'%top_column)
                            sql += '+'.join(sum_sql)
                            sql += ' from stats where '
                            sql += filter_sql
                            sql += ''.join(' group by %s'%top_column)
                            try :
                                #sql: select CLF_XXX, sum(h0)+sum(h1)+..+sum(hr) from stats where filter group by CLF_XXX
                                logging.debug('sql:%r and filter :%r',sql,filter_list)
                                ret = cursor.execute(sql, filter_list).fetchall() # [('xxx',10),(xx,90),...]
                                top_result[top_column] = ret
                           # result.append(top_result) # [{'CLF_XXX':[('xx',10),('xx',20)....]},{'CLF_XXX':[('xxx':10),..]},...]
                            except Exception,e:
                                logging.exception('top query error %r',e)
                    sql = 'select '
                result.append(top_result) # [{'CLF_XXX':[('xx',10),('xx',20)....],'CLF_XXX':[('xxx':10),..], ....}]

        else: # filter is none
            if is_ldap_filter: # ldap_filter is True
                sql = 'select '+ ','.join(reportStats[stats_name]['logFields'])+', '
                #sql += '+'.join(ll)
                sql += '+'.join(sum_ldap)
                sql += ' from stats '
                try:
                    ret = cursor.execute(sql).fetchall()
                    result.extend(ret)
                except Exception,e:
                    logging.exception('query error %r',e)
            else: #is_ldap_filter is Non
                facet_query = len(reportStats[stats_name]['logFields'])>= 4 and 'CLF_DeviceName' in top_name
                sql = 'select '
                for top_column in top_name:
                    # maybe query first facet page
                    # select columnID, (h0)+..+(hr) from facet where columIDType = top_name
                    # columnID,columnIDType
                    if facet_query:
                        sql += 'columnID, '
                        sql += '+'.join(sum_ldap)
                        sql += ' from facet '
                        sql += 'where columnIDType=?'
                        try:
                            logging.debug('sql:%r',sql)
                            ret = cursor.execute(sql,[top_column]).fetchall()
                            top_result[top_column] = ret
                        except Exception,e:
                            logging.exception('have no any filter query error %r',e)
                    else:
                        sql += ''.join('%s,'%top_column)
                        sql += '+'.join(sum_sql)
                        sql += ' from stats'
                        sql += ''.join(' group by %s'%top_column)
                        try:
                            logging.debug('sql:%r',sql)
                            ret = cursor.execute(sql).fetchall()
                            top_result[top_column] = ret
                            #result.append(top_result)
                        except Exception,e:
                            logging.exception('have no any filter query error %r',e)
                    sql = 'select '
                result.append(top_result)
        return result


    def __query_Trending(self,db,cursor,top_name,filter,is_ldap_filter, start_end_list,stat_name):
        # maybe we will not  ignore is_ldap_filter, but we will should consider filter parameter

        if self.merge_mode != 'day':
            logging.error('supported the trending report to be create in merge_mode of day ')
            return None
        result = []
        trend_ret = {}
        sum_sql, sum_ldap = self.__query_sum_time_parse(start_end_list)
        filter_sql, filter_list = self.__query_filter_parse(filter)
        sql = 'select '

        # sql: select sum(h0), sum(h1),...,sum(hr) from stats ..
        if filter and len (filter) > 0:
            if is_ldap_filter:
                sql += ','.join(is_ldap_filter)  # is_ldap_filter = ['CLF_Username','CLF_ClientIP']
                sql += ','
                sql += ','.join(sum_sql)
                sql += ' from stats'
                sql += ' where'+filter_sql
                sql += ' group by '
                sql += ','.join(is_ldap_filter)
                try:
                    # select ldap_filter, sum(h0),...,sum(hr) from stats where filter group by ldap_filter
                    logging.debug('trending fiter with ldap sql:%r',sql)
                    ret = cursor.execute(sql,filter_list).fetchall()
                    result.extend(ret) # return: [(use,ip,1,2,3,..),(use,ip,..),....]
                except Exception,e:
                    logging.exception(' trend Ture query error %r',e)
            else:
                if stat_name == 'facet_internet_access' and len(filter) == 1 and 'CLF_DeviceName' in filter:
                    sql += ','.join(sum_sql)
                    sql += ' from facet'
                    sql += ' where '+filter_sql+' and columnIDType=?'
                    try:
                        # select sum(h0),...,sum(hr) from stats where filter
                        logging.debug('trending fiter with no ldap sql:%r',sql)
                        ret = cursor.execute(sql,filter_list+['CLF_UserName']).fetchall()
                        result.extend(ret) # return: [(1,2,3,..)]
                    except Exception,e:
                        logging.exception(' trend Ture query error %r',e)
                else:
                    sql += ','.join(sum_sql)
                    sql += ' from stats'
                    sql += ' where '+filter_sql
                    try:
                        # select sum(h0),...,sum(hr) from stats where filter
                        logging.debug('trending fiter with no ldap sql:%r',sql)
                        ret = cursor.execute(sql,filter_list).fetchall()
                        result.extend(ret) # return: [(1,2,3,..)]
                    except Exception,e:
                        logging.exception(' trend Ture query error %r',e)
        else:
            if is_ldap_filter:
                sql += ','.join(is_ldap_filter)  # is_ldap_filter = ['CLF_Username','CLF_ClientIP']
                sql += ','
                sql += ','.join(sum_sql)
                sql += ' from stats'
                sql += ' group by '
                sql += ','.join(is_ldap_filter)
                try:
                    logging.debug('trending no fiter with ldap sql:%r',sql)
                    ret = cursor.execute(sql).fetchall()
                    result.extend(ret) # return: [(use,ip,1,2,3,..),(use,ip,..),....]
                except Exception,e:
                    logging.exception(' trend Ture query error %r',e)
            else:
                sql += ','.join(sum_sql)
                if stat_name == 'facet_internet_access':
                    sql += ' from facet where columnIDType=?'
                else:
                    sql += ' from stats'
                try:
                    logging.debug('trending no fiter with no ldap sql:%r',sql)
                    if stat_name == 'facet_internet_access':
                        ret = cursor.execute(sql,['CLF_UserName']).fetchall()
                    else:
                        ret = cursor.execute(sql).fetchall()
                    result.extend(ret)
                except Exception,e:
                    logging.exception(' trend Ture query error %r',e)

        logging.debug("Trending result: %r", result)
        return result


    def __query_multi_Trending(self,db,cursor,top_name,filter,is_ldap_filter, start_end_list,stat_name):
       # maybe we will not ignore is_ldap_filter, but we will should consider filter parameter
        if self.merge_mode != 'day':
            logging.error('supported the trending report to be create in merge_mode of day ')
            return None
        result = []
        sum_sql, sum_ldap = self.__query_sum_time_parse(start_end_list)
        filter_sql, filter_list = self.__query_filter_parse(filter)
        # select CLF_XXX, CLF_XXX, sum(h)... from stats group by CLF_XXX, CLF_XXX
        if len(top_name) > 1:
            logging.error('MutiTrending masterColumID have more than 1 ')
        sql = 'select '+top_name[0]+','
        sql += ','.join(sum_sql)
        sql += ' from stats'
        if filter:
            if is_ldap_filter:
                sql = 'select '+','.join(is_ldap_filter)+','  # is_ldap_filter = ['CLF_Username','CLF_ClientIP']
                sql += top_name[0]+','
                sql += ','.join(sum_sql)
                sql += ' from stats'
                sql += ' where'+filter_sql
                sql += ' group by '+','.join(is_ldap_filter) +',' +top_name[0]
                try:
                    # select ldap_filter, top_name
                    logging.debug('multi-trending fiter with ldap sql:%r',sql)
                    ret = cursor.execute(sql,filter_list).fetchall()
                    result.extend(ret) # return: [(user,ip ,top_name[0],,1,2,3,4,),....]
                except Exception,e:
                    logging.exception(' trend Ture query error %r',e)
            else:
                sql = 'select '+top_name[0]+','
                sql += ','.join(sum_sql)
                sql += ' from stats'
                sql += ' where'+filter_sql
                sql += ' group by '+ top_name[0]
                try:
                    logging.debug('multi-trending  filter with no ldap sql:%r',sql)
                    ret = cursor.execute(sql,filter_list).fetchall()
                    result.extend(ret) # return: [(top_name[0],,1,2,3,4,),....]
                except Exception,e:
                    logging.exception(' trend Ture query error %r',e)
        else:
            if is_ldap_filter:
                sql = 'select '+','.join(is_ldap_filter)+','  # is_ldap_filter = ['CLF_Username','CLF_ClientIP']
                sql += top_name[0]+','
                sql += ','.join(sum_sql)
                sql += ' from stats'
                sql += ' group by '+','.join(is_ldap_filter) +',' +top_name[0]
                try:
                    logging.debug('multi-trending no fiter with ldap sql:%r',sql)
                    ret = cursor.execute(sql).fetchall()
                    result.extend(ret) # return: [(user,ip ,top_name[0],,1,2,3,4,),....]
                except Exception,e:
                    logging.exception(' trend Ture query error %r',e)
            else:
                sql = 'select '+top_name[0]+','
                sql += ','.join(sum_sql)
                sql += ' from stats'
                sql += ' group by '+ top_name[0]
                try:
                    logging.debug('multi-trending  no filter with no ldap sql:%r',sql)
                    ret = cursor.execute(sql).fetchall()
                    result.extend(ret)
                except Exception,e:
                    logging.exception(' trend Ture query error %r',e)
        return result



    def __query(self,db,cursor,top_name,filter,is_ldap_filter, start_end_list,stat_name, reportType):
    # table, eg: user, domain, cat,h0,h1,.......,h23 , filter = {'xxx':[xxx,xxx],'xxx',[xxx,xxxx]}
    #sql = 'select user, sum(h0)+sum(h1)+..+sum(hr) from stats (summary) while cat= ? and domain = group by user ?')
        if not isinstance(top_name,list):
            logging.debug('__query top_name is not list')
            top_name = [top_name]
        
        if reportType == reportDef.reportType['trending']:
            return self.__query_Trending(db,cursor,top_name,filter,is_ldap_filter, start_end_list,stat_name)
        elif reportType == reportDef.reportType['multiTrending']:
            return self.__query_multi_Trending(db,cursor,top_name,filter,is_ldap_filter, start_end_list,stat_name)
        else:
            return self.__query_topOrMulti_top(db,cursor,top_name,filter,is_ldap_filter, start_end_list,stat_name)

    def split_time(self,cid,timeBegin,timeEnd,timezone):
    # split time three, for 2012/1/12:12:00---2012/1/23:12:00 ---> 2012/1/12:12 to 2012/1/12:59 2012/1/13:0:00-2012/1/22:23:59
    #2012/1/23:00:00 to 2012/1/23:12:00
        first_time=[]
        second_time=[]
        three_time=[]
        timeBegin = timeBegin/3600*3600
        timeEnd = timeEnd/3600*3600
        hour, ts = self.__getTimestampStartDayTime(cid,timeBegin,timezone)
        if hour==0:
            second_time.append(timeBegin)
        else:
            first_time.append(timeBegin)
            oneday = datetime.timedelta(days=1)
            ts_d = datetime.datetime.fromtimestamp(ts,timezone)
            next_d = ts_d+oneday
            next_timeBegin = calendar.timegm(next_d.utctimetuple())
            first_time.append(next_timeBegin-1)
            second_time.append(next_timeBegin)
        end_hour,end_ts = self.__getTimestampStartDayTime(cid,timeEnd,timezone)
        if end_hour == 23:
            second_time.append(timeEnd)
        else:
            second_time.append(end_ts-1)
            three_time.append(end_ts)
            three_time.append(timeEnd)

        return first_time,second_time,three_time

    def __find_trending(self, coll,companyID, timeBegin, timeEnd,month_table_time,stat_name,top_name,filter,is_ldap_filter,reportType):
        zone,_ = self.__get_localzone()
        ret2 = {}
        if month_table_time and month_table_time < timeEnd:
            first_time,second_time,three_time = self.split_time(companyID,timeBegin,month_table_time,zone)
            if three_time:
                three_time[1] = timeEnd
            else:
                three_time = [second_time[1]+1,timeEnd]
        else:
            first_time,second_time,three_time = self.split_time(companyID,timeBegin,timeEnd,zone)
        if len(second_time) == 2:
            ret2 = self.find_stats_month(companyID, second_time[0], second_time[1], stat_name,top_name,filter,is_ldap_filter,reportType,zone)
        if len(first_time) == 2:
            ret1 = self.find_stats(companyID, first_time[0], first_time[1], stat_name,top_name,filter,is_ldap_filter,reportType)
            _,first_timestamp = self.__getTimestampStartDayTime(companyID,timeBegin,zone)
            if reportType == reportDef.reportType['trending']:
                total=0
                # if is_ldap_filter is not Null , ret1 = {(xxx,xx):{ts1:xx, ts2:xxx}....}
                if ret1:
                    if is_ldap_filter:
                        tmp_ret1 = {}
                        for userOrip, subresult in ret1.iteritems():
                            total = 0
                            for ts, count in subresult.iteritems(): # ret1= {ts:xx,ts:xx,...}
                                total += int(count)
                            tmp_ret1[userOrip] = {first_timestamp:total}
                            if ret2.has_key(userOrip):
                                ret2[userOrip].update(tmp_ret1[userOrip])
                            else:
                                ret2[userOrip] = tmp_ret1[userOrip]
                    else:
                        for ts, count in ret1.iteritems(): # ret1= {ts:xx,ts:xx,...}
                            total += int(count)
                        ret1 = {first_timestamp:total}
                        ret2.update(ret1)
            else: #reportDef.reportType['multiTrending'] {(user,ip,ooo):{ts1:xx,ts2:xx...},(use,ip,ooo):{ts1:xx,ts2:xx}....}
                tmp_ret = {}
                for key, ts_dict in ret1.iteritems():
                    total = 0
                    for ts, count in ts_dict.iteritems():
                        total += int(count)
                        tmp_ret[key]={first_timestamp:total}
                        ret1 = tmp_ret

        if len(three_time) == 2:
            ret3 = self.find_stats(companyID, three_time[0], three_time[1], stat_name,top_name,filter,is_ldap_filter,reportType)
            _,three_timestamp = self.__getTimestampStartDayTime(companyID,three_time[0],zone)
            if reportType == reportDef.reportType['trending']:
                total=0
                if ret3:
                    # if is_ldap_filter is not Null , ret1 = {(xxx,xx):{ts1:xx, ts2:xxx}....}
                    if is_ldap_filter:
                        # ret3 = {(xxx,xx):{ts1:xx, ts2:xxx}....}
                        for userOrip, subresult in ret3.iteritems():
                            tmp_ret3 = {}
                            for ts, count in subresult.iteritems(): # ret1= {ts:xx,ts:xx,...}
                                _,day_timestamp = self.__getTimestampStartDayTime(companyID,ts,zone)
                                if tmp_ret3.has_key(day_timestamp):
                                    tmp_ret3[day_timestamp] += count
                                else:
                                    tmp_ret3[day_timestamp] = count
                            if ret2.has_key(userOrip):
                                ret2[userOrip].update(tmp_ret3)
                            else:
                                ret2[userOrip] = tmp_ret3
                    else:
                        tmp_ret3 = {}
                        for ts, count in ret3.iteritems(): # ret1= {ts:xx,ts:xx,...}
                            _,day_timestamp = self.__getTimestampStartDayTime(companyID,ts,zone)
                            if tmp_ret3.has_key(day_timestamp):
                                tmp_ret3[day_timestamp] += count
                            else:
                                tmp_ret3[day_timestamp] = count
                        ret2.update(tmp_ret3)

            else: #reportDef.reportType['multiTrending'] {(user,ip,ooo):{ts1:xx,ts2:xx...},(use,ip,ooo):{ts1:xx,ts2:xx}....}
                tmp_ret = {}
                for key, ts_dict in ret3.iteritems():
                    total = 0
                    for ts, count in ts_dict.iteritems():
                        total += int(count)
                        tmp_ret[key]={first_timestamp:total}
                ret3 = tmp_ret
        #merge all result
        if reportType == reportDef.reportType['trending']:
            return ret2
        else:
            for key1, ts_value in ret1.iteritems():
                if ret2.has_key(key1):
                    ret2[key].update(ts_value)
                else:
                    ret2.update({key1:ts_value})
            for key3, ts_value in ret3.iteritems():
                if ret2.has_key(key3):
                    ret2[key].update(ts_value)
                else:
                    ret2.update({key3:ts_value})
            return ret2

    def __find_stats_by_monthtable(self,coll,companyID, timeBegin, timeEnd, month_table_time,stat_name,top_name,filter,is_ldap_filter,reportType):
        ret = []
        zone,_ = self.__get_localzone()
        # splite time , first time , second_time , and three time
        if month_table_time and month_table_time < timeEnd:
            first_time,second_time,three_time = self.split_time(companyID,timeBegin,month_table_time,zone)
            if three_time:
                three_time[1] = timeEnd
            else:
                three_time = [second_time[1]+1,timeEnd]
        else:
            first_time,second_time,three_time = self.split_time(companyID,timeBegin,timeEnd,zone)

        if len(first_time) == 2:
            ret1 = self.find_stats(companyID, first_time[0], first_time[1], stat_name,top_name,filter,is_ldap_filter,reportType)
            if ret1:
                ret.extend(ret1)
        if len(second_time) == 2:
            ret2 = self.find_stats_month(companyID, second_time[0], second_time[1], stat_name,top_name,filter,is_ldap_filter,reportType,zone)
            if ret2:
                ret.extend(ret2)
        if len(three_time) == 2:
            ret3 = self.find_stats(companyID, three_time[0], three_time[1], stat_name,top_name,filter,is_ldap_filter,reportType)
            if ret3:
                ret.extend(ret3)
        return ret


    def find(self,coll,companyID, timeBegin, timeEnd, stat_name,top_name,filter,is_ldap_filter,reportType,scheduled_time_filter=None):
        if coll == CLFCfg.CLF_DB_DASHBOARD_STATS:
            return self.find_dashboard(companyID, timeBegin, timeEnd, stat_name,top_name,filter,is_ldap_filter)
        if coll == CLFCfg.CLF_DB_STATS:
            month_time = self.manage_cur.execute('select day_time from  time')

            time_range = 0
            month_table_time = 0
            #= (timeEnd - timeBegin)/24/3600
            zone,_ = self.__get_localzone()
            #if 0:
            lock = None
            if os.path.exists(self.month_lock_dbfile):
                lock_db = sqlite3.connect(self.month_lock_dbfile,check_same_thread = False)
                lock_cur = lock_db.cursor()
                lock = lock_cur.execute('select month_lock_status from lock').fetchall()
                lock_db.close()
                month_time = self.manage_cur.execute('select day_time from  time').fetchall()
                if month_time:
                    if timeEnd >= month_time[0][0] and month_time[0][0] > timeBegin:
                        time_range = (month_time[0][0]-timeBegin)/24/3600
                        month_table_time = month_time[0][0]
                    if timeEnd < month_time[0][0]:
                        time_range = (timeEnd - timeBegin)/24/3600
                    if month_time[0][0] < timeBegin:
                        time_range = 0

            #self.month_lock_db.commit()
            if lock:
                is_month_lock = int(lock[0][0])
            else:
                is_month_lock = 1

            is_scheduled_time_always = True
            if scheduled_time_filter and scheduled_time_filter.has_key('timeType'):
                is_scheduled_time_always = scheduled_time_filter['timeType'] == 0
                
            if is_scheduled_time_always and not is_month_lock and time_range >= 5 and (reportType != reportDef.reportType['trending'] and  reportType != reportDef.reportType['multiTrending']):
                #time_range = (timeEnd - timeBegin)/24/3600
                timeEnd = timeEnd-1
                return self.__find_stats_by_monthtable(coll,companyID, timeBegin, timeEnd,month_table_time,stat_name,top_name,filter,is_ldap_filter,reportType)
                #return self.find_stats_month(companyID, timeBegin, timeEnd, stat_name,top_name,filter,is_ldap_filter,reportType,zone)
            elif is_scheduled_time_always and not is_month_lock and time_range >= 5 and (reportType == reportDef.reportType['trending'] or reportType == reportDef.reportType['multiTrending']):
                timeEnd = timeEnd-1
                return self.__find_trending(coll,companyID, timeBegin, timeEnd,month_table_time,stat_name,top_name,filter,is_ldap_filter,reportType)
                #return ret1+ret2+ret3
            else:
                timeEnd = timeEnd-1
                result = self.find_stats(companyID, timeBegin, timeEnd, stat_name,top_name,filter,is_ldap_filter,reportType,scheduled_time_filter)
                
                if (timeEnd - timeBegin)/3600/24 >= 5 and reportType == reportDef.reportType['trending'] :
                    # should merge all result data, hour ---> day
                    # if is_ldap_filter = [xx, xxx]
                    # should return {(user,ip):{ts1:xx, ts2:xx, ... ts3:xxx}}
                    tmp_result = {}
                    if is_ldap_filter:
                        for userOrip, subresult in result.iteritems():
                            sub_tmp_result = {}
                            for res_time, res_count in subresult.iteritems():
                                _,day_timestamp = self.__getTimestampStartDayTime(companyID,res_time,zone)
                                if sub_tmp_result.has_key(day_timestamp):
                                    sub_tmp_result[day_timestamp] += res_count
                                else:
                                    sub_tmp_result[day_timestamp] = res_count
                            tmp_result[userOrip] = sub_tmp_result
                    else:
                        for res_time, res_count in result.iteritems():
                            _,day_timestamp = self.__getTimestampStartDayTime(companyID,res_time,zone)
                            if tmp_result.has_key(day_timestamp):
                                tmp_result[day_timestamp] += res_count
                            else:
                                tmp_result[day_timestamp] = res_count
                    return tmp_result
                else:
                    return result

                #return self.find_stats(companyID, timeBegin, timeEnd, stat_name,top_name,filter,is_ldap_filter,reportType)


    """
     find ts: xxx by some parameters .
     first should make sure month table time postion
     if can not query from month table, should handle error
    """
    def find_stats_time(self,coll,companyID, timeBegin, timeEnd, stat_name,filter,mode):
        # check month table time postion
        resutlt  = {}
        month_time = self.manage_cur.execute('select day_time from  time')
        month_time = month_time.fetchone()
        lock = None
        if os.path.exists(self.month_lock_dbfile):
            lock_db = sqlite3.connect(self.month_lock_dbfile,check_same_thread = False)
            lock_cur = lock_db.cursor()
            lock = lock_cur.execute('select month_lock_status from lock').fetchall()
            lock_db.close()
                    #self.month_lock_db.commit()
        if lock:
            is_month_lock = int(lock[0][0])
        else:
            is_month_lock = 1

        if mode == 'day' and is_month_lock == 0 and month_time[0] > timeBegin :
            zone,_ = self.__get_localzone()
            if month_time[0] >= timeEnd:
                first_time,second_time,three_time = self.split_time(companyID,timeBegin,timeEnd,zone)
                ret1 = self.find_stats(companyID, first_time[0], first_time[1], stat_name,[],filter,False,reportDef.reportType['trending'])
                if ret1:
                    resutlt['hour'] = ret1
                ret2 = self.find_stats_month(companyID, second_time[0], second_time[1], stat_name,[],filter,False,reportDef.reportType['trending'],zone)
                if ret2:
                    resutlt['day'] = ret2
                ret3 = self.find_stats(companyID, three_time[0], three_time[1], stat_name,[],filter,False,reportDef.reportType['trending'])
                if ret3:
                    if ret1:
                        resutlt['hour'].update(ret3)
                    else:
                        resutlt['hour'] = ret3
            else:
                first_time,second_time,three_time = self.split_time(companyID,timeBegin,month_time[0],zone)
                if three_time:
                    three_time[1] = timeEnd
                else:
                    three_time = [second_time[1]+1,timeEnd]

                ret1 = self.find_stats(companyID, first_time[0], first_time[1], stat_name,[],filter,False,reportDef.reportType['trending'])
                if ret1:
                    resutlt['hour'] = ret1
                ret2 = self.find_stats_month(companyID, second_time[0], second_time[1], stat_name,[],filter,False,reportDef.reportType['trending'],zone)
                if ret2:
                    resutlt['day'] = ret2
                ret3 = self.find_stats(companyID, three_time[0], three_time[1], stat_name,[],filter,False,reportDef.reportType['trending'])
                if ret3:
                    if ret1:
                        resutlt['hour'].update(ret3)
                    else:
                        resutlt['hour'] = ret3
        else:
            ret1 = self.find_stats(companyID, timeBegin, timeEnd, stat_name,[],filter,False,reportDef.reportType['trending'])
            resutlt['hour'] = ret1
        return resutlt

    # you can call the API to query browse time stats
    # browse time db only store stats data of 5 min and store time flag which is used as calculating time flag
    # so browse time db has two tables :
    # table1 : cid, stats_name, ts                # ts indicate postion of calculating stats
    # table2 : cid , ts ,statsname ,dur ,stats    # this table store 5 min stats data
    def find2(self,coll, filter,result={}):
        # {'col': CLFCfg.CLF_StatsInterval, 'op': '==', 'value': 300}, {CLFCfg.CLF_CompanyID: 1, CLFCfg.CLF_TimeStamp: 1, CLFCfg.CLF_StatsName: 1}
        # first parse filter parameters
        filter_list= []
        filter_list_value= []
        b_query_ts = False
        for subfilter in filter:
            if subfilter['col'] == CLFCfg.CLF_StatsInterval and subfilter['value'] == 3600:
                b_query_ts = True
                continue
            if subfilter['op'] == '==' or subfilter['op'] == 'eq':
                s = ''.join('%s=?'%subfilter['col'])
                filter_list.append(s)
                filter_list_value.append(subfilter['value'])

            else:
                logging.info('not support the parameters: %r',subfilter)
        result_value=[]
        for k, value in result.iteritems():  # result = {ts:1} or {'stats':1}
            if value > 0:
                result_value.append(k)

        if len(result_value) > 0:
            sql_result = ','.join(result_value)
        else:
            sql_result = '*'
        if len(filter_list) > 0 :
            sql_filter = ' where '
            sql_filter += 'and '.join(filter_list)
        else:
            sql_filter=''
        if b_query_ts: # select ts from timestamp where cid =? and stats_name = ?
            sql =  'select '+sql_result+ ' from timestamp ' +sql_filter
        else:
            sql =  'select '+sql_result+ ' from stats ' +sql_filter  # select stats from stats where cid=? and ts =? and statsname=? dur=?
        db_name = self.stats_db_path + 'browse_time.db'
        query_result=[]
        try:
            if not self.browse_time_db:
                db = sqlite3.connect(db_name)
                cur =  db.cursor()
                if len(filter_list) > 0 :
                    ret = cur.execute(sql,filter_list_value)
                else:
                    ret = cur.execute(sql)
                self.browse_time_db = db
            else:
                db = self.browse_time_db
                cur =  db.cursor()
                if len(filter_list) > 0 :
                    ret = cur.execute(sql,filter_list_value)
                else:
                    ret = cur.execute(sql)
               # ret = [(xxx,xxx,xxx),(xxx,xxx,xxx)], query_result= [{xxx:xxx},...],result=['','','']
            for item in ret:  # 1. {ts:134xxxxx}  2. stats:[[xxx,xxx,90],[xxx,xxx,80],...,]
                tmp_dict = {}
                tmp_dict.update(itertools.izip(result_value,item))
                query_result.append(tmp_dict)

        except Exception, e:
            logging.exception('find2 browse time stats  data from DB fail %r',e)
        return query_result

    def find_dashboard(self,companyID, timeBegin, timeEnd, stat_name,top_name,filter,is_ldap_filter):
        db_name = ''.join('%s%s_%s.db'%(self.dashboard_path,companyID,stat_name))
        dur = filter['dur']
        db = sqlite3.connect(db_name)
        results = []
        try :
            cur = db.cursor() # cid,ts,stats_name,dur CLFCfg.CLF_CompanyID,CLFCfg.CLF_TimeStamp,CLFCfg.CLF_StatsName,CLFCfg.CLF_StatsInterval
            sql = ''.join('select %s, stats from dashboard where %s=? and %s=? and %s=? and %s between ? and ?'%(CLFCfg.CLF_TimeStamp,CLFCfg.CLF_CompanyID,CLFCfg.CLF_StatsName,CLFCfg.CLF_StatsInterval,CLFCfg.CLF_TimeStamp))
            ret = cur.execute(sql,[companyID,stat_name,dur,timeBegin,timeEnd]).fetchall()
            #ret = cur.execute('select ts,doc from dashboard where cid=? and stats_name=? and dur=? and ts between ? and ?',[companyID,stat_name,dur,timeBegin,timeEnd]).fetchall()
            if len(ret) > 0:
                for item in ret:
                    tmp_dict = {CLFCfg.CLF_Stats:item[1],CLFCfg.CLF_TimeStamp:item[0],CLFCfg.CLF_StatsZip:0}
                    results.append(tmp_dict)
            db.close()
            return results
        except Exception,e:
            logging.exception('find dashboard have error %r',e)
            return []

    def __calculate_stats_result(self,cid,day_start,start_end_list,stats_data,top_result,trend_result,reportType,ldap_filter,month_mode=None):
        # top_result = [], trend_result = {}
        if not isinstance(top_result,list) or not isinstance(trend_result,dict) or day_start < 0:
            logging.error('calculate  parameters are error: cid:%r,day:%r,result:%r',cid,day_start,top_result)
            return None
        if reportType == reportDef.reportType['trending']: #and month_mode is None : #self.merge_mode == 'day':   # should query time (t0,t1, t2,t3,....), reportType == reportDef.reportType['trending']
            stats = {}
            # result = [{'timestamp1':xx, 'timestamp2':xxx,...}]
            tmp_timestamp = []
            shift = 0
            for s_hour,e_hour in start_end_list:
                for i in range(s_hour,e_hour+1):  #
                    if month_mode:
                        timestamp = self.__getTimestampStartDayUTCtimestamp(cid,day_start,i)
                    else:
                        timestamp = self.__getTimestampStartHourUTCtimestamp(cid,day_start,i)
                    if ldap_filter:  # stats_data = [(user,ip,1,2,3,4,5,...),(user,ip,1,2,3,..)]
                        tmp_timestamp.append(timestamp)
                    else:
                        if len(stats_data) > 0: # stats_data = [(1,2,3,4,5,...)]
                            if stats_data[0][i-s_hour+shift]:
                                stats[timestamp] = stats_data[0][i-s_hour+shift] # stats = {'ts':xxx, 'ts1':xx,....}
                shift += (e_hour - s_hour + 1)
            if ldap_filter:
                # trend_result = {(user,ip):{ts1:xxx,ts2...},(user,ip):{ts1:xx,ts2:xx}}
                num_filter = len(ldap_filter)
                for stats_item in stats_data: # stats_data = [(user,ip,1,2,3,4,5,...),(user,ip,1,2,3,..)]
                    if trend_result.has_key(stats_item[0:num_filter]):
                        sub_stats_item = trend_result[stats_item[0:num_filter]]
                        ts_list =  stats_item[num_filter:]
                        sub_stats_item.update(itertools.izip(tmp_timestamp,ts_list))
                    else:
                        new_item = {}
                        new_item.update(itertools.izip(tmp_timestamp,stats_item[num_filter:]))
                        trend_result[stats_item[0:num_filter]] = new_item  # trend_result = {(user,ip):{ts1:xx,ts2:xx...},(use,ip):{ts1:xx,ts2:xx}....}
            else:
                # {ts1:xxx,ts2:xx,....}
                trend_result.update(stats)

        #elif reportType == reportDef.reportType['trending'] and month_mode:


            #return stats
        elif reportType == reportDef.reportType['multiTrending']: #and month_mode is None : # self.merge_mode == 'day':
            logging.debug('query len :%r',len(stats_data[0]))
            stats = {}
            tmp_timestamp = []
            shift = 0
            for s_hour,e_hour in start_end_list:
                for i in range(s_hour,e_hour+1):
                    if month_mode:
                        timestamp = self.__getTimestampStartDayUTCtimestamp(cid,day_start,i)
                    else:
                        timestamp = self.__getTimestampStartHourUTCtimestamp(cid,day_start,i)
                    #timestamp = self.__getTimestampStartHourUTCtimestamp(cid,day_start,i)
                        # return result: clf_xxx,sum(h0),sun(h1),sun(h2),... group by clf_xxx
                        #[(u'sales rep1', 1, 6), (u'sales rep10', 0, 6), (u'sales rep11', 0, 3),(u'sales rep2', 0, 0),
                        # (u'sales rep3', 0, 0),(u'sales rep4', 0, 3),(u'sales rep5', 0, 3),(u'sales rep6', 0, 3),
                        # (u'sales rep7', 0, 3),(u'sales rep8', 0, 0),(u'sales rep9', 0, 3)]
                    if ldap_filter:  # stats_data = [(user,ip,1,2,3,4,5,...),(user,ip,1,2,3,..)]
                        tmp_timestamp.append(timestamp)
                    else:
                        for entry in stats_data:
                            if entry[1+i-s_hour+shift]:
                                tmp_dict = {timestamp:entry[1+i-s_hour+shift]}
                                if trend_result.has_key(entry[0]):
                                    trend_result[entry[0]].update(tmp_dict) # stats = {clf_xxx:{'timestamp':xx,....}}
                                else:
                                    trend_result[entry[0]] = tmp_dict
                shift += (e_hour - s_hour + 1)
                
            if ldap_filter:
                # trend_result = {(user,ip,ooo):{ts1:xx,ts2:xx...},(use,ip,ooo):{ts1:xx,ts2:xx}....}
                num_filter = len(ldap_filter)+1
                for stats_item in stats_data: # stats_data = [(user,ip,ooo,1,2,3,4,5,...),(user,ip,ooo,1,2,3,..)]
                    if trend_result.has_key(stats_item[0:num_filter]):
                        sub_stats_item = trend_result[stats_item[0:num_filter]]
                        ts_list =  stats_item[num_filter:]
                        sub_stats_item.update(itertools.izip(tmp_timestamp,ts_list))
                    else:
                        new_item = {}
                        new_item.update(itertools.izip(tmp_timestamp,stats_item[num_filter:]))
                        trend_result[stats_item[0:num_filter]] = new_item
                # result = [stats]
        else:
            # for top or muti-top merge report:
            top_result.extend(stats_data) #
        return top_result,trend_result

    def find_stats_month(self,companyID, timeBegin, timeEnd, stat_name,top_name,filter,is_ldap_filter, reportType,zone=None):
        statsName = stat_name
        # timeBegin and timeEnd is UTC time, should translate to Time zone
        startTime = timeBegin
        endTime = timeEnd
        #************************************************************
        #1.calculate day default
        whole_unit = 31
        whole_unit_start = 1
        start_hour, day_start = self.__getTimestampStartMonthTime(companyID,timeBegin,zone)
        end_hour, day_end = self.__getTimestampStartMonthTime(companyID,timeEnd,zone)
        #************************************************************
        #companyID = companyID

        result = []
        logging.debug('cid %r, starttime %r, endtime %r, stats_name %r', companyID,startTime,endTime,statsName)
        trend_ret = {}

        if day_start == day_end: # query in the whole day or one month
            day_time  = day_start
            s_hour = start_hour
            e_hour = end_hour
            db_name = ''.join('%s%s_%s_%r_month.db'%(self.stats_db_path,companyID,statsName,day_time))
            if not os.path.exists(db_name):
                logging.debug('%s database is not exists',db_name)
                return result
            db = sqlite3.connect(db_name)
            cur = db.cursor()
            ret = self.__query(db,cur,top_name,filter,is_ldap_filter,[(s_hour,e_hour)],stat_name,reportType)
            db.close()
            #__query(self,db,cursor,top_name,filter,is_ldap_filter, start,end)
            self.__calculate_stats_result(companyID,day_start,[(s_hour,e_hour)],ret,result,trend_ret,reportType,is_ldap_filter,zone)

        else :  # query time span a doy
            # query all stats database from manage db , and timestamp in [day_start --- day_end]
            find_stats_db_SQL = 'select db_name, ts from month_statsname where cid=? and stats_name=? and ts between ? and ?'
            ret_db = self.manage_cur.execute(find_stats_db_SQL,[companyID,stat_name,day_start,day_end]).fetchall()
            if len(ret_db) == 0:
                logging.error('have no stats , check stats database')
                if (reportType == reportDef.reportType['trending'] or reportType == reportDef.reportType['multiTrending']):
                    return trend_ret
                else :
                    return result
            #query result: [(db_name, ts), (db_name1, ts1).....] = ret_db
            for db_ts in ret_db:
                if db_ts[1] == day_start:
                    s_hour = start_hour
                    e_hour = self.__getMonthRange(companyID, day_start, zone)

                    db_name = db_ts[0]
                    if not os.path.exists(db_name):
                        logging.debug('%s database is not exists',db_name)
                        continue
                    logging.debug('database is %r', db_name)
                    db = sqlite3.connect(db_name)
                    cur = db.cursor()
                    ret = self.__query(db,cur,top_name,filter,is_ldap_filter,[(s_hour,e_hour)],stat_name,reportType)
                    db.close()
                    self.__calculate_stats_result(companyID,db_ts[1],[(s_hour,e_hour)],ret,result,trend_ret,reportType,is_ldap_filter,zone)


                elif db_ts[1] == day_end:
                    s_hour = whole_unit_start
                    e_hour = end_hour
                    db_name =  db_ts[0]
                    if not os.path.exists(db_name):
                        logging.debug('%s database is not exists',db_name)
                        continue
                    logging.debug('database is %r', db_name)
                    db = sqlite3.connect(db_name)
                    cur = db.cursor()
                    ret = self.__query(db,cur,top_name,filter,is_ldap_filter,[(s_hour,e_hour)],stat_name,reportType)
                    db.close()
                    self.__calculate_stats_result(companyID,db_ts[1],[(s_hour,e_hour)],ret,result,trend_ret,reportType,is_ldap_filter, zone)

                else:
                    s_hour = whole_unit_start
                    e_hour = self.__getMonthRange(companyID, db_ts[1], zone)

                    #print companyID,statsName,day_start + i*whole_time
                    db_name = db_ts[0]
                   # print 'db_name is %r'%db_name
                    if not os.path.exists(db_name):
                        logging.debug('%s database is not exists',db_name)
                        continue
                    logging.debug('database is %s', db_name)
                    db = sqlite3.connect(db_name)
                    cur = db.cursor()
                    ret = self.__query(db,cur,top_name,filter,is_ldap_filter,[(s_hour,e_hour)],stat_name,reportType)
                    db.close()
                    self.__calculate_stats_result(companyID,db_ts[1],[(s_hour,e_hour)],ret,result,trend_ret,reportType,is_ldap_filter,zone)

        if (reportType == reportDef.reportType['trending'] or reportType == reportDef.reportType['multiTrending'])  and self.merge_mode == 'day':
            return trend_ret
        else :
            return result


    def find_stats(self,companyID, timeBegin, timeEnd, stat_name,top_name,filter,is_ldap_filter, reportType, scheduled_time_filter=None):
        statsName = stat_name
        # timeBegin and timeEnd is UTC time, should translate to Time zone
        startTime = timeBegin
        endTime = timeEnd
        #************************************************************
        if (reportType == reportDef.reportType['trending'] or reportType == reportDef.reportType['multiTrending']) and self.merge_mode != 'day':
            logging.error('find stats merge mode or parameters is error')
            return []
        
        if scheduled_time_filter and scheduled_time_filter['timeType'] != 0 and self.merge_mode != 'day':
            logging.error('find_stats cannot handle work/leisure time filter when merge mode is not day.')
            return []

        if not scheduled_time_filter or scheduled_time_filter['timeType'] == 0:
            #1.calculate day default
            if self.merge_mode == 'day':
                whole_unit = 23
                whole_unit_start = 0
                start_hour, day_start = self.__getTimestampStartDayTime(companyID,timeBegin)
                end_hour, day_end = self.__getTimestampStartDayTime(companyID,timeEnd)
            elif self.merge_mode == 'month':
                whole_unit = 31
                whole_unit_start = 1
                start_hour, day_start = self.__getTimestampStartMonthTime(companyID,timeBegin)
                end_hour, day_end = self.__getTimestampStartMonthTime(companyID,timeEnd)
            else:
                whole_unit = 7
                whole_unit_start = 1
            #************************************************************
            #companyID = companyID

            result = []
            logging.debug('cid %r, starttime %r, endtime %r, stats_name %r', companyID,startTime,endTime,statsName)
            trend_ret = {}

            if day_start == day_end: # query in the whole day or one month
                day_time  = day_start
                s_hour = start_hour
                e_hour = end_hour
                db_name = ''.join('%s%s_%s_%r.db'%(self.stats_db_path,companyID,statsName,day_time))
                if not os.path.exists(db_name):
                    logging.debug('%s database is not exists',db_name)
                    if (reportType == reportDef.reportType['trending'] or reportType == reportDef.reportType['multiTrending'])  and self.merge_mode == 'day':
                        return trend_ret
                    else :
                        return result
                db = sqlite3.connect(db_name)
                cur = db.cursor()
                ret = self.__query(db,cur,top_name,filter,is_ldap_filter,[(s_hour,e_hour)],stat_name,reportType)
                db.close()
                #__query(self,db,cursor,top_name,filter,is_ldap_filter, start,end)
                self.__calculate_stats_result(companyID,day_start,[(s_hour,e_hour)],ret,result,trend_ret,reportType,is_ldap_filter)

            else :  # query time span a doy
                # query all stats database from manage db , and timestamp in [day_start --- day_end]
                find_stats_db_SQL = 'select db_name, ts from statsname where cid=? and stats_name=? and ts between ? and ?'
                ret_db = self.manage_cur.execute(find_stats_db_SQL,[companyID,stat_name,day_start,day_end]).fetchall()
                if len(ret_db) == 0:
                    logging.error('have no stats , check stats database')

                    if (reportType == reportDef.reportType['trending'] or reportType == reportDef.reportType['multiTrending'])  and self.merge_mode == 'day':
                        return trend_ret
                    else :
                        return result


                #query result: [(db_name, ts), (db_name1, ts1).....] = ret_db
                for db_ts in ret_db:
                    if db_ts[1] == day_start:
                        s_hour =  start_hour
                        e_hour = whole_unit
                        db_name = db_ts[0]
                        if not os.path.exists(db_name):
                            logging.debug('%s database is not exists',db_name)
                            continue
                        logging.debug('database is %r', db_name)
                        db = sqlite3.connect(db_name)
                        cur = db.cursor()
                        ret = self.__query(db,cur,top_name,filter,is_ldap_filter,[(s_hour,e_hour)],stat_name,reportType)
                        db.close()
                        self.__calculate_stats_result(companyID,db_ts[1],[(s_hour,e_hour)],ret,result,trend_ret,reportType,is_ldap_filter)


                    elif db_ts[1] == day_end:
                        s_hour = whole_unit_start
                        e_hour = end_hour
                        db_name =  db_ts[0]
                        if not os.path.exists(db_name):
                            logging.debug('%s database is not exists',db_name)
                            continue
                        logging.debug('database is %r', db_name)
                        db = sqlite3.connect(db_name)
                        cur = db.cursor()
                        ret = self.__query(db,cur,top_name,filter,is_ldap_filter,[(s_hour,e_hour)],stat_name,reportType)
                        db.close()
                        self.__calculate_stats_result(companyID,db_ts[1],[(s_hour,e_hour)],ret,result,trend_ret,reportType,is_ldap_filter)

                    else:
                        s_hour = whole_unit_start
                        e_hour = whole_unit
                        #print companyID,statsName,day_start + i*whole_time
                        db_name = db_ts[0]
                       # print 'db_name is %r'%db_name
                        if not os.path.exists(db_name):
                            logging.debug('%s database is not exists',db_name)
                            continue
                        logging.debug('database is %s', db_name)
                        db = sqlite3.connect(db_name)
                        cur = db.cursor()
                        ret = self.__query(db,cur,top_name,filter,is_ldap_filter,[(s_hour,e_hour)],stat_name,reportType)
                        db.close()
                        self.__calculate_stats_result(companyID,db_ts[1],[(s_hour,e_hour)],ret,result,trend_ret,reportType,is_ldap_filter)

            if (reportType == reportDef.reportType['trending'] or reportType == reportDef.reportType['multiTrending'])  and self.merge_mode == 'day':
                return trend_ret
            else :
                return result
        else:   # work/leisure time report
            result = []
            logging.debug('cid %r, starttime %r, endtime %r, stats_name %r', companyID,startTime,endTime,statsName)
            trend_ret = {}
            t_pair = clf_datetime.getTimePair(companyID, timeBegin, timeEnd, scheduled_time_filter)
            
            for ts in t_pair:
                db_name = ''.join('%s%s_%s_%r.db'%(self.stats_db_path,companyID,statsName,ts))
                if not os.path.exists(db_name):
                    logging.debug('%s database is not exists',db_name)
                    continue
                logging.debug('database is %r', db_name)
                db = sqlite3.connect(db_name)
                cur = db.cursor()
                ret = self.__query(db,cur,top_name,filter,is_ldap_filter,t_pair[ts],stat_name,reportType)
                db.close()
                
                self.__calculate_stats_result(companyID,ts,t_pair[ts],ret,result,trend_ret,reportType,is_ldap_filter)

            if (reportType == reportDef.reportType['trending'] or reportType == reportDef.reportType['multiTrending'])  and self.merge_mode == 'day':
                return trend_ret
            else :
                return result
            

    def remove(self, coll, filter):     # we should implement this interface (TODO:)
    #                        self.__stats_engine.cleanupStats(cur_ts + dur, [  {'col': CLFCfg.CLF_TimeStamp, 'op': '<', 'value': expiredTime}
     #                           {'col': CLFCfg.CLF_CompanyID, 'op': 'eq', 'value': cid},
      #                          {'col': CLFCfg.CLF_StatsName, 'op': 'eq', 'value': statName},
       #                         {'col': CLFCfg.CLF_StatsInterval, 'op': '==', 'value': 300}])
      # cid TEXT, ts INTEGER,statsname TEXT,dur INTEGER,doc TEXT
        filter_list= []
        filter_list_mon= []
        filter_list_value = []
        filter_list_value_mon = []
        db_name = None
        b_remove_browse_time_stats = False
        b_para_error =  False
        statName = None

        zone,_ = self.__get_localzone()
        cid = '0'
        for item in filter:
            if item['col'] == CLFCfg.CLF_CompanyID:
                cid = item['value']
            elif item['col'] == CLFCfg.CLF_TimeStamp:
                ts = item['value']
        day,mon_ts = self.__getTimestampStartMonthTime(cid, ts, zone)

        for item in filter:
            if item['col'] == CLFCfg.CLF_StatsInterval and item['value'] == 300:
            # remove in
                b_remove_browse_time_stats =  True
                if item['op'] != '==':
                    b_para_error =True
                break
        if coll == CLFCfg.CLF_DB_STATS:
             for item in filter:
                if item['col'] == CLFCfg.CLF_StatsInterval and item['value'] == 300:
                    filter_list.append(item['col']+'=?')
                    filter_list_value.append(300)
                elif item['col'] == CLFCfg.CLF_StatsName:
                    statName = item['value']
                    filter_list.append(item['col']+'=?')
                    filter_list_value.append(item['value'])
                    filter_list_mon.append(item['col']+'=?')
                    filter_list_value_mon.append(item['value'])
                elif item['col'] == CLFCfg.CLF_TimeStamp:
                    if item['op'] == 'eq' or item['op'] == '==':
                        filter_list.append(item['col']+'=?')
                        filter_list_value.append(item['value'])
                        filter_list_mon.append(item['col']+'=?')
                        filter_list_value_mon.append(item['value'])
                    else:
                        filter_list.append(item['col']+item['op']+'?')
                        filter_list_value.append(item['value'])
                        filter_list_mon.append(item['col']+item['op']+'?')
                        filter_list_value_mon.append(mon_ts)
                else:
                    if item['op'] == 'eq' or item['op'] == '==':
                        filter_list.append(item['col']+'=?')
                        filter_list_value.append(item['value'])
                        filter_list_mon.append(item['col']+'=?')
                        filter_list_value_mon.append(item['value'])
                    else:
                        filter_list.append(item['col']+item['op']+'?')
                        filter_list_value.append(item['value'])
                        filter_list_mon.append(item['col']+item['op']+'?')
                        filter_list_value_mon.append(item['value'])

             if not b_para_error and b_remove_browse_time_stats:
                db_name = self.stats_db_path + 'browse_time.db'
                db = sqlite3.connect(db_name)
                cur = db.cursor()
                try:
                    sql = 'delete from stats where '+ ' and '.join(filter_list)
                    cur.execute(sql,filter_list_value)
                    db.commit()
                except Exception,e:
                    logging.exception('delete browse time stats error %r',e)
             else: # delete stats , but first query db's information, and delete them and than delete DB file
                 find_stats_db_SQL = 'select db_name from statsname where '+' and '.join(filter_list)
                 del_stats_db_SQL = 'delete from statsname where '+' and '.join(filter_list)
                 logging.debug('manage file DB delete query SQL:%r',find_stats_db_SQL)
                 ret_db = self.manage_cur.execute(find_stats_db_SQL,filter_list_value).fetchall()
                 if ret_db: # delete db file name
                     logging.debug('will delete all db file name :%r',ret_db)
                     self.manage_cur.execute(del_stats_db_SQL,filter_list_value)
                     self.manage_db.commit()
                     for db_name in ret_db:
                         logging.debug(db_name)
                         delete_cmd = 'rm -rf ' +  db_name[0]
                         os.system(delete_cmd)

                 #delete month table data
                 logging.debug("Purge month DB %d: before day: %d", mon_ts, day)
                 find_stats_db_SQL = 'select db_name from month_statsname where '+' and '.join(filter_list_mon)
                 del_stats_db_SQL = 'delete from month_statsname where '+' and '.join(filter_list_mon)
                 logging.debug('manage month file DB delete query SQL: %r, %r',find_stats_db_SQL, filter_list_value_mon)
                 ret_db = self.manage_cur.execute(find_stats_db_SQL,filter_list_value_mon).fetchall()
                 if ret_db: # delete db file name
                     logging.debug('will delete all db file name :%r',ret_db)
                     self.manage_cur.execute(del_stats_db_SQL,filter_list_value_mon)
                     self.manage_db.commit()
                     for db_name in ret_db:
                         logging.debug("Remove month file: %r", db_name)
                         delete_cmd = 'rm -rf ' +  db_name[0]
                         os.system(delete_cmd)

                 #delete in this month
                 if day > 1:
                    db_name_sql = 'select db_name,stats_name from month_statsname where ts=?'
                    result = self.manage_cur.execute(db_name_sql,[mon_ts]).fetchall()
                    if result:
                        for rel in result:
                            conn = sqlite3.connect(rel[0])

                            update_set = ''
                            for hx in xrange(1,day):
                                update_set += 'h' + str(hx) + '=0,'
                            update_set = update_set[0:-1]   #remove last ','
                            cur = conn.cursor()
                            update_stats_db_SQL = 'update facet set ' + update_set
                            logging.debug('Update month: %r', update_stats_db_SQL)
                            cur.execute(update_stats_db_SQL)
                            update_stats_db_SQL = 'update stats set ' + update_set
                            cur.execute(update_stats_db_SQL)

                            if rel[1] == 'facet_internet_access':
                                update_stats_db_SQL = 'update stats_2 set ' + update_set
                                cur.execute(update_stats_db_SQL)
                            conn.commit()
                            conn.close()

        if coll == CLFCfg.CLF_DB_DASHBOARD_STATS:
            #        self.db.remove(CLFCfg.CLF_DB_DASHBOARD_STATS, [
            #            {'col': CLFCfg.CLF_CompanyID, 'op': 'eq', 'value': companyID},
            #            {'col': CLFCfg.CLF_TimeStamp, 'op': '<', 'value': expiredTime}])
            cid = None
            expiredTime = 0
            for item in filter:
                if item['col'] == CLFCfg.CLF_CompanyID:
                    cid = item['value']
                if item['col'] == CLFCfg.CLF_TimeStamp:
                    expiredTime = item['value']
            if not cid:
                logging.error('del dashboard para error, %r',filter)
                return
            sql = ''.join('delete from dashboard where %s<?'%CLFCfg.CLF_TimeStamp)
            for dash_name in dashboardStats:
                db_name = ''.join('%s%s_%s.db'%(self.dashboard_path,cid,dash_name))
                try:
                    db = sqlite3.connect(db_name)
                    cur =  db.cursor()
                    logging.debug('exeute sql %r',sql)
                    cur.execute(sql,[expiredTime])
                    db.commit()
                    db.close()
                except Exception, e:
                    logging.exception('Remove dashboard stats failed:%s', e)

    def __get_localzone(self,_root='/'):
        """Tries to find the local timezone configuration.
        This method prefers finding the timezone name and passing that to pytz,
        over passing in the localtime file, as in the later case the zoneinfo
        name is unknown.

        The parameter _root makes the function look for files like /etc/localtime
        beneath the _root directory. This is primarily used by the tests.
        In normal usage you call the function without parameters."""

        tzenv = os.environ.get('TZ')
        if tzenv:
            return _tz_from_env(tzenv)

        # CentOS has a ZONE setting in /etc/sysconfig/clock,
        # OpenSUSE has a TIMEZONE setting in /etc/sysconfig/clock and
        # Gentoo has a TIMEZONE setting in /etc/conf.d/clock
        # We look through these files for a timezone:

        zone_re = re.compile('\s*ZONE\s*=\s*\"')
        timezone_re = re.compile('\s*TIMEZONE\s*=\s*\"')
        end_re = re.compile('\"')

        for filename in ('etc/sysconfig/clock', 'etc/conf.d/clock'):
            tzpath = os.path.join(_root, filename)
            if not os.path.exists(tzpath):
                continue
            with open(tzpath, 'rt') as tzfile:
                data = tzfile.readlines()

            for line in data:
                 # Look for the ZONE= setting.
                match = zone_re.match(line)
                if match is None:
                     # No ZONE= setting. Look for the TIMEZONE= setting.
                    match = timezone_re.match(line)
                if match is not None:
                    # Some setting existed
                    line = line[match.end():]
                    etctz = line[:end_re.search(line).start()]

                    # We found a timezone
                    return pytz.timezone(etctz), etctz

        raise pytz.UnknownTimeZoneError('Can not find any timezone configuration')

    def __create_month_db(self, timestamp, stats_name,cid, month_db = False):
        if month_db:
            db_name = ''.join('%s%s_%s_%r.db'%(self.stats_db_path,cid,stats_name,timestamp))
        else:
            db_name = ''.join('%s%s_%s_%r_month.db'%(self.stats_db_path,cid,stats_name,timestamp))
        try:
            if not self.memoryDB.has_key(db_name):
                db = sqlite3.connect(db_name)
                cur =  db.cursor()

                cur.execute(self.table[stats_name]['stats'])
                # for facet table
                cur.execute(self.table[stats_name]['facet'])

                self.manage_cur.execute('REPLACE INTO statsname VALUES(?,?,?,?)', [cid,stats_name,timestamp,db_name])
                self.manage_db.commit()
                old_db=''.join('%s%s_%s'%(self.stats_db_path,cid,stats_name))
                keys  = self.memoryDB.keys()
                for k in keys:
                    m = re.match(old_db, k)
                    if m is not None:
                        old_db_name = self.memoryDB.pop(k)
                        logging.debug('old_db:%r, old_db_name:%r',old_db, old_db_name)
                        old_db_name.close()

                self.memoryDB[db_name] = db
        except Exception, e:
            logging.exception('create memoryDB fail %r',e)
        return self.memoryDB[db_name]

    """
    content : stats data: [(xxx, xxx, 12, 23),(xxx,xxx, 23,45).....]
    colculate_type :
         2: stats data [(xxx,xxx,12,23),(xx,xx,23,45)] stats in 2 day , calculate the second day and the first day
         1: stats data [(xxx,xxx,12),(xxx,xxx,23)...] stats in one day
        -1: stats data [(xxx,xxx,12,23),(xx,xx,23,45)] stats in 2 day , calculate the second day
        -2: stats data [(xxx,xxx,12,23),(xx,xx,23,45)] stats in 2 day , calculate the first day
    """
    def __update_month_table(self, content ,day, db,stats_name,cid,calculate_type=1):
        t_hour = day
        if calculate_type == 1:
            stats_index = -1
            count_index = -1
        if calculate_type == 2:
            stats_index = -2
            count_index = [-2,-1]
        else:
            stats_index = -2
            count_index = calculate_type
        #t_hour = day
        s2 = self.S3
        sql2 = self.REPACE_SQL3
        facet_repace_sql =self.FACET_REPACE_SQL3
        stats2_repace_sql = self.STATS2_REPACE_SQL3

        filter = self.table[stats_name]['filter']
        facet_list = self.table[stats_name]['facet_list']

        sql = ''.join('SELECT %s FROM stats WHERE %s'%(s2, filter)) #CLF_Devicename
        facet_sql = ''.join('SELECT %s FROM facet WHERE columnID = ? and columnIDType= ? and CLF_DeviceName=? '%s2)
        stats2_sql = ''.join('SELECT %s FROM stats_2 WHERE CLF_DeviceName = ? and CLF_URLCategory=? and CLF_CustomURLCategory=? and columnID=? and columnIDType=?'%s2)
        logging.debug('db_update first query sql:%r',sql)
        insert_list = []
        facet_repace_sql +=',?,?,?)'
        stats2_repace_sql += ',?,?,?,?,?)'
        try:
            c = db.cursor()
            logging.info('insert month table data ******* start ')
            for item  in content: # [(xxx, xxx, 12,23),(xx,xx,24,25),...]
                result = c.execute(sql,list(item[:stats_index])).fetchone()  # k = (xx,xx,)  query old stats data and then merge stats
                if calculate_type == 2: # t_hour is list
                    new_counter = self.calculate_conter(t_hour,[item[-2],item[-1]],result)
                else: # t_hour is value
                    new_counter = self.calculate_conter(t_hour,item[count_index],result)
                repace_sql = sql2 + ',?'*len(item[:stats_index]) + ')'
                logging.debug('repace_sql:%r',repace_sql)
                logging.debug('new counter:%r',new_counter)
                #stats_data.append(list(item[:stats_index])+new_counter)
                c.execute(repace_sql,list(item[:stats_index])+new_counter)

                #update facet table CLF_DeviceName and 'CLF_DeviceName' in k
                if len(item[:stats_index]) >=4 and  'CLF_DeviceName' in reportStats[stats_name]['logFields']:
                    deviceID =  reportStats[stats_name]['logFields'].index('CLF_DeviceName')
                    columnIds = list(item[:stats_index])
                    for i, columnId in enumerate(columnIds):
                        #facet_sql = ''.join('SELECT %s FROM stats WHERE %s'%(s2, filter))
                        result = c.execute(facet_sql,[columnId,facet_list[i],columnIds[deviceID]]).fetchone()  # k = (xx,xx,)
                        if calculate_type == 2: # t_hour is list
                            new_facet_counter = self.calculate_conter(t_hour,[item[-2],item[-1]],result)
                        else: # t_hour is value
                           new_facet_counter = self.calculate_conter(t_hour,item[count_index],result)
                        #new_facet_counter = self.calculate_conter(t_hour,item[count_index],result)
                        #logging.debug('repace_sql:%r',repace_sql)
                        logging.debug('new counter:%r',new_facet_counter)

                        c.execute(facet_repace_sql,[columnId,facet_list[i],columnIds[deviceID]]+new_facet_counter)

                if stats_name == 'facet_internet_access':
                #CLF_DeviceName TEXT,CLF_URLCategory TEXT ,
                # CLF_CustomURLCategory TEXT ,columnID TEXT,columnIDType TEXT
                    result = c.execute(stats2_sql,[item[0],item[4],item[5],item[1],'CLF_UserName']).fetchone()
                    if calculate_type == 2: # t_hour is list
                        new_stats2_counter = self.calculate_conter(t_hour,[item[-2],item[-1]],result)
                    else: # t_hour is value
                        new_stats2_counter = self.calculate_conter(t_hour,item[count_index],result)
                    c.execute(stats2_repace_sql,[item[0],item[4],item[5],item[1],'CLF_UserName']+new_stats2_counter)

                    result = c.execute(stats2_sql,[item[0],item[4],item[5],item[2],'CLF_ClientIP']).fetchone()
                    if calculate_type == 2: # t_hour is list
                        new_stats2_counter = self.calculate_conter(t_hour,[item[-2],item[-1]],result)
                    else: # t_hour is value
                        new_stats2_counter = self.calculate_conter(t_hour,item[count_index],result)
                    c.execute(stats2_repace_sql,[item[0],item[4],item[5],item[2],'CLF_ClientIP']+new_stats2_counter)

                    #result = c.execute(stats2_sql,[item[0],item[6],item[7],item[4],'CLF_ServerIP']).fetchone()
                    #new_stats2_counter = self.calculate_conter(t_hour,item[-1],result)
                    #c.execute(stats2_repace_sql,[item[0],item[6],item[7],item[4],'CLF_ServerIP']+new_stats2_counter)

                    result = c.execute(stats2_sql,[item[0],item[4],item[5],item[3],'CLF_Domain']).fetchone()
                    if calculate_type == 2: # t_hour is list
                        new_stats2_counter = self.calculate_conter(t_hour,[item[-2],item[-1]],result)
                    else: # t_hour is value
                        new_stats2_counter = self.calculate_conter(t_hour,item[count_index],result)
                    c.execute(stats2_repace_sql,[item[0],item[4],item[5],item[3],'CLF_Domain']+new_stats2_counter)
            logging.info('insert month table data &&&&& end  ')
        except Exception, e:
            logging.exception('update data to DB fail %r',e)


    def __calculate_month_stats(self, ts_dbs): # ts_dbs : cid TEXT, stats_name TEXT, ts INTEGER,db_name TEXT
        # ts is UTC time , db_file is db_path=cid_statsname_ts.db
        if not ts_dbs :
            logging.info('__calculate_month_stats para is error!')
        conn = sqlite3.connect(ts_dbs[3])
        cur = conn.cursor()
        localtimezone,_ = self.__get_localzone()
        # calculate ts in where month and month start time
        # for example :       1361289600(UTC) in Asia/Shanghai  is 2013/2/20
        # ts_dbs[1]:stats_name
        columnIDs = self.table[ts_dbs[1]]['facet_list']
        query_day_sql = 'select ' + ','.join(columnIDs)
        try:
            day, monthTime = self.__getTimestampStartMonthTime(ts_dbs[0], ts_dbs[2], localtimezone)
            db_name = ''.join('%s%s_%s_%r_month.db'%(self.stats_db_path,ts_dbs[0],ts_dbs[1],monthTime)) # cid_statsname_ts_month.db
            stats_conn = sqlite3.connect(db_name)

            stats_cur = stats_conn.cursor()
            page_size = stats_cur.execute('PRAGMA page_size').fetchone()[0]
            if page_size != 32768: #32K
                stats_cur.execute('PRAGMA page_size = 32768;')
                stats_conn.commit()
                stats_cur.execute('VACUUM')
                stats_conn.commit()


            stats_cur.execute(self.table[ts_dbs[1]]['month_stats']) # create month table
            stats_cur.execute(self.table[ts_dbs[1]]['month_facet']) # create month facet table
            stats_cur.execute(self.table[ts_dbs[1]]['month_stats2']) # create stats_2 table
            self.__createIndexForDB(ts_dbs[1],stats_conn,stats_cur)

            #find next_day
            sum_hour = []
            sum_hour_next = []
            for i in range(24):
                next_day, next_monthTime = self.__getTimestampStartMonthTime(ts_dbs[0], ts_dbs[2]+i*3600, localtimezone)
                if next_day != day or monthTime != next_monthTime:
                    break
                s = ''.join('h%d'%i)
                sum_hour.append(s)

            if i != 24:  # a part if stats data is in day and another part is in next_day
                for j  in range(i,24):
                    s = ''.join('h%d'%j)
                    sum_hour_next.append(s)
                query_day_sql += ','+'+'.join(sum_hour) +','+'+'.join(sum_hour_next) + ' from stats'
                logging.debug('__calculate_month_stats query day sql:%r',query_day_sql)
                # query select xxx, xxx, sum(h0)+..+sum(hr), sum(hr)+..+sum(h23)
                ret = cur.execute(query_day_sql) # ret = [(xxx, xxx, 12,13),(xx,xx,24,25),...]
                if monthTime != next_monthTime:
                    # calculate first day
                    self.__update_month_table(ret ,day, stats_conn,ts_dbs[1],ts_dbs[0],-2)  # stats is not difference month table database
                    self.manage_cur.execute('REPLACE INTO month_statsname VALUES(?,?,?,?)', [ts_dbs[0],ts_dbs[1],monthTime,db_name])
                    self.manage_db.commit()
                    stats_cur.execute('VACUUM')
                    stats_conn.commit()
                    stats_conn.close()

                    db_name = ''.join('%s%s_%s_%r_month.db'%(self.stats_db_path,ts_dbs[0],ts_dbs[1],next_monthTime))
                    monthTime = next_monthTime
                    stats_conn = sqlite3.connect(db_name)
                    stats_cur = stats_conn.cursor()
                    page_size = stats_cur.execute('PRAGMA page_size').fetchone()[0]
                    if page_size != 32768: #32K
                        stats_cur.execute('PRAGMA page_size = 32768;')
                        stats_conn.commit()
                        stats_cur.execute('VACUUM')
                        stats_conn.commit()

                    stats_cur.execute(self.table[ts_dbs[1]]['month_stats'])
                    stats_cur.execute(self.table[ts_dbs[1]]['month_facet'])
                    stats_cur.execute(self.table[ts_dbs[1]]['month_stats2'])
                    self.__createIndexForDB(ts_dbs[1],stats_conn,stats_cur)
                    # calculate the second day
                    ret = cur.execute(query_day_sql) # ret = [(xxx, xxx, 12,13),(xx,xx,24,25),...]
                    self.__update_month_table(ret ,next_day, stats_conn,ts_dbs[1],ts_dbs[0],-1)
                    stats_cur.execute('VACUUM')
                    stats_conn.commit()

                else:
                    self.__update_month_table(ret ,[day,next_day], stats_conn,ts_dbs[1],ts_dbs[0],2)
                    stats_cur.execute('VACUUM')
                    stats_conn.commit()
            else: # stats data is in day
                query_day_sql += ','+'+'.join(sum_hour) + ' from stats'
                logging.info('__calculate_month_stats query day sql:%r',query_day_sql)
                ret = cur.execute(query_day_sql) # ret = [(xxx, xxx, 12),(xx,xx,24),...]
                self.__update_month_table(ret, day, stats_conn,ts_dbs[1],ts_dbs[0],1)
                stats_cur.execute('VACUUM')
                stats_conn.commit()

            conn.close()

            stats_conn.close()
            self.manage_cur.execute('REPLACE INTO month_statsname VALUES(?,?,?,?)', [ts_dbs[0],ts_dbs[1],monthTime,db_name])
            self.manage_db.commit()
        except Exception,e:
            logging.exception('__calculate_month_stats query day table error')


    def do_month_data(self,cid = '0',timeBegin = 0, timeEnd = 0):
        # first query manage file time table , and determine month table time and day time
        lock_db = sqlite3.connect(self.month_lock_dbfile)
        lock_cur = lock_db.cursor()
        lock_cur.execute('CREATE TABLE IF NOT EXISTS lock (month_lock_status INTEGER)')
        try:
            time_table = self.manage_cur.execute('select * from time').fetchall()
            if time_table:
                day_time  = time_table[0][0]
                day_hour = time_table[0][1]
                month_time = time_table[0][2]
                time_zone = time_table[0][3]
                lock_cur.execute('update lock set month_lock_status =?',[1])
                lock_db.commit()
            else:
                day_time  = 0
                day_hour = 0
                month_time = 0
                time_zone = 'UTC'
                self.manage_cur.execute('insert into time values (?,?,?,?)',[day_time,day_hour,month_time,time_zone])
                self.manage_db.commit()
                lock_cur.execute('insert into lock values(?)',[1])
                lock_db.commit()

            _,local_zone = self.__get_localzone()
            if time_zone != local_zone:
                day_time  = 0
                day_hour = 0
                month_time = 0
                time_zone = local_zone
                # clear up all stats information in month table

        except Exception,e:
            logging.exception('query time table error:%r',e)
            lock_cur.execute('update lock set month_lock_status =?',[0])
            lock_db.commit()
            lock_db.close()
            return
        # query all stats db information of manage_db file (ts >= day_time)
        try:
            # query all stats_name
            stats_names = self.manage_cur.execute('select stats_name from statsname  group by stats_name').fetchall()
            need_calcu_month_table_list = []
            #ts_line = 0
            now = 0
            if timeBegin >=0  and timeEnd > 0 and timeEnd > timeBegin:
                table_start_hour, table_start_time  =self.__getTimestampStartDayTime(cid,timeBegin)
                table_end_hour, table_end_time  =self.__getTimestampStartDayTime(cid,timeEnd)
                if table_end_time == table_start_time:
                    if table_end_hour - table_start_hour < 23:
                        logging.error('do month table time is error, timeBegin :%r, timeEnd:%r',timeBegin,timeEnd)
                    else:
                        if day_time >= table_start_time:
                            logging.info('do month table have been done, day_time:%r, begin time:%r',day_time,timeBegin)
                            return
                        else:
                            now = timeEnd
                else: # end_time > start time
                    now = table_end_time-1
            else:
                now = int(time.time())
                now = (now/3600/24)*3600*24-3600*24

            max_ts = 0
            if stats_names and now > day_time :
                for _stat_name in stats_names:
                 # ts_dbs = [(cid TEXT, stats_name TEXT, ts INTEGER,db_name TEXT), ]
                    ts_dbs = self.manage_cur.execute('select * from statsname where stats_name=? and ts between ? and ? order by ts ASC',[_stat_name[0],day_time +1, now]).fetchall()
                    logging.debug('get statsname table : %r',ts_dbs)
                    need_calcu_month_table_list.extend(ts_dbs)
                        #if ts_dbs and ts_dbs[-1][2] > ts_line:
                        #   ts_line = need_calcu_month_table_list[-1][2] #ts
                        #ts_line =
                for ts_db in need_calcu_month_table_list:
                    self.__calculate_month_stats(ts_db)
                    if max_ts < ts_db[2]:
                        max_ts = ts_db[2]
            else:
                logging.info('stats name is not exsit')
                lock_cur.execute('update lock set month_lock_status =?',[0])
                lock_db.commit()
                lock_db.close()
                return
            if max_ts > 0 :
                self.manage_cur.execute('update time set day_time=?  where day_time=?',[max_ts,day_time])
                self.manage_cur.execute('update time set time_zone=? where day_time=?',[time_zone,max_ts])
                #self.manage_cur.execute('update time set day_hour=? where day_time = ?',[24,max_ts])
                self.manage_db.commit()

        except Exception,e:
            logging.exception('query stats_name table error:%r',e)
            lock_cur.execute('update lock set month_lock_status =?',[0])
            lock_db.commit()
            lock_db.close()
            return
        lock_cur.execute('update lock set month_lock_status =?',[0])
        lock_db.commit()
        lock_db.close()

    def connect(self, db_name):
        pass

#    CLF_DB_MONGODB = 0
#    CLF_DB_SOLR =1
#    CLF_DB_LOCAL_DISK= 2
#    CLF_DB_STATS_SQLITE = 3
class logBlockStatus(object):
    def __init__(self):
        CLFCfg.init()
        self.stats_db_path = CLFCfg.CLF_STATS_STORE_PATH
        self.MongoDBUri = CLFCfg.CLF_DB_MONGODB_URI
        self.stats_db   = CLFCfg.CLF_STATS_DB_TYPE
        if self.stats_db == CLFCfg.CLF_DB_MONGODB:
            self.conn = pymongo.Connection( host=self.MongoDBUri,
                                   _connect=False,
                                   read_preference=pymongo.ReadPreference.SECONDARY)
            self.db = self.conn[db_name]
        if self.stats_db == CLFCfg.CLF_DB_STATS_SQLITE:
            #self.stats_db_path = CLFCfg.CLF_STATS_STORE_PATH+'stats/'
            #self.dashboard_path = CLFCfg.CLF_STATS_STORE_PATH+'dashboard/'
            #if not os.path.exists(self.stats_db_path):
            #os.makedirs(self.stats_db_path)
            if not os.path.exists(CLFCfg.CLF_STATS_STORE_PATH):
                os.makedirs(CLFCfg.CLF_STATS_STORE_PATH)
            db_name = CLFCfg.CLF_STATS_STORE_PATH + 'logblockstatus'
            self.conn = sqlite3.connect(db_name, timeout=10)
            self.db =  self.conn.cursor()
            self.db.execute('CREATE TABLE IF NOT EXISTS logblock (filename TEXT, cid TEXT, blockurl TEXT,summary_status INTEGER, stats_status INTEGER,dashboard_status INTEGER)')
        #self.status =


    def saveLogBlockIntitalStatus(self,filename,cid,blockurl,status):
        if not filename or not isinstance(status,int):
            return
        if self.stats_db == CLFCfg.CLF_DB_MONGODB:
            pass

        if self.stats_db == CLFCfg.CLF_DB_STATS_SQLITE:
            if status == 0: # should insert this file status
                self.db.execute('INSERT INTO logblock VALUES(?,?,?,?,?,?)',[filename,cid,blockurl,0,0,0])
                self.conn.commit()
            else:
                try:
                    if status == 1:  # update logblock summary status
                        self.db.execute('UPDATE logblock SET summary_status=? where filename=?',[1,filename])
                    elif status == 2: # update logblock stats status
                        self.db.execute('UPDATE logblock SET stats_status=? where filename=?',[1,filename])
                    elif status == 3: # update logblock dashboard status
                        self.db.execute('UPDATE logblock SET dashboard_status=? where filename=?', [1,filename])
                    else:
                        pass
                    self.conn.commit()
                except Exception,e:
                    logging.error('saveLogBlockIntitalStatus error :%r',e)


    def getLogBlockStatus(self, filename):
        if not filename:
            return
        if self.stats_db == CLFCfg.CLF_DB_MONGODB:
            pass
        if self.stats_db == CLFCfg.CLF_DB_STATS_SQLITE:
            try :
                if filename == '*':
                    ret = self.db.execute('select filename,summary_status,stats_status,dashboard_status from logblock')
                else:
                    ret = self.db.execute('select filename, summary_status,stats_status,dashboard_status from logblock where filename=?',[filename])
                return ret
            except Exception,e:
                logging.error('getLogBlockStatus error :%r',e)
                return None


    def delLogBlockStatus(self, filenames):
        if not filenames:
            return
        if self.stats_db == CLFCfg.CLF_DB_MONGODB:
            pass
        if self.stats_db == CLFCfg.CLF_DB_STATS_SQLITE:
            files = []
            for filename in filenames:
                files.append([filename])

            self.db.executemany('delete from logblock where filename=?',files)
            self.conn.commit()

    def getUndoneLogBlocks(self):
        if self.stats_db == CLFCfg.CLF_DB_MONGODB:
            return []
        if self.stats_db == CLFCfg.CLF_DB_STATS_SQLITE:
            try :
                ret = self.db.execute('select filename,cid,blockurl,summary_status,stats_status,dashboard_status from logblock where summary_status=0 or stats_status=0 or dashboard_status=0')
                return ret
            except Exception,e:
                logging.error('getUndoneLogBlocks error :%r',e)
                return []


if __name__ == '__main__':
    pass
