#!/usr/bin/env python
# -*- coding: utf-8 -*-

import pytz
import datetime
import calendar
import os
import re
import logging

def get_localzone(_root='/'):
    """Tries to find the local timezone configuration.
    This method prefers finding the timezone name and passing that to pytz,
    over passing in the localtime file, as in the later case the zoneinfo
    name is unknown.

    The parameter _root makes the function look for files like /etc/localtime
    beneath the _root directory. This is primarily used by the tests.
    In normal usage you call the function without parameters."""

    tzenv = os.environ.get('TZ')
    if tzenv:
        return _tz_from_env(tzenv)

    # CentOS has a ZONE setting in /etc/sysconfig/clock,
    # OpenSUSE has a TIMEZONE setting in /etc/sysconfig/clock and
    # Gentoo has a TIMEZONE setting in /etc/conf.d/clock
    # We look through these files for a timezone:

    zone_re = re.compile('\s*ZONE\s*=\s*\"')
    timezone_re = re.compile('\s*TIMEZONE\s*=\s*\"')
    end_re = re.compile('\"')

    for filename in ('etc/sysconfig/clock', 'etc/conf.d/clock'):
        tzpath = os.path.join(_root, filename)
        if not os.path.exists(tzpath):
            continue
        with open(tzpath, 'rt') as tzfile:
            data = tzfile.readlines()

        for line in data:
             # Look for the ZONE= setting.
            match = zone_re.match(line)
            if match is None:
                 # No ZONE= setting. Look for the TIMEZONE= setting.
                match = timezone_re.match(line)
            if match is not None:
                # Some setting existed
                line = line[match.end():]
                etctz = line[:end_re.search(line).start()]

                # We found a timezone
                return pytz.timezone(etctz), etctz

    raise pytz.UnknownTimeZoneError('Can not find any timezone configuration')

# timestamp is UTC , return : hour is Timezone local hour, dayTime is start time (0:00:0) of UTC
def getTimestampStartDayTime(cid,timestamp,timezone=None):
    # first translate timestamp into Timezone's datetime
    timestamp = (timestamp/3600)*3600
    if timezone:
        date_zone = datetime.datetime.fromtimestamp(timestamp, timezone)
    else:
        date_zone = datetime.datetime.fromtimestamp(timestamp, pytz.utc)
    hour = date_zone.hour # get hour of timezone
    #tmp_date = datetime.datetime(date_zone.year,date_zone.month,date_zone.day,0,0)
    tmp_date = date_zone.replace(hour=0)

    dayTime = calendar.timegm(tmp_date.utctimetuple())
    return hour, dayTime

    
def getTimePairLocalZone(cid,ts1, ts2,scheduled_time_filter,timezone):
    # first translate timestamp into Timezone's datetime
    ts1 = (ts1/3600)*3600
    ts2 = (ts2/3600)*3600

    date1 = datetime.datetime.fromtimestamp(ts1, timezone)
    date2 = datetime.datetime.fromtimestamp(ts2, timezone)
    
    hour1 = date1.hour # get hour of timezone
    tmp_date1 = date1.replace(hour=0)
    dayTime1 = calendar.timegm(tmp_date1.utctimetuple())
    hour2 = date2.hour # get hour of timezone
    tmp_date2 = date2.replace(hour=0)
    dayTime2 = calendar.timegm(tmp_date2.utctimetuple())
    
    t_pair = {}
    while dayTime1 <= dayTime2:
        if not t_pair.has_key(dayTime1):
            t_pair[dayTime1] = []
        h2 = 23 if dayTime1 != dayTime2 else hour2
        weekday = date1.isoweekday()
        if scheduled_time_filter['WorkTime_Day'][(weekday+1)%7-1] == '1':  #work day
            if scheduled_time_filter['timeType'] == 1:  #work time report
                if hour1 <= scheduled_time_filter['WorkTime_PM_End'] and h2 > scheduled_time_filter['WorkTime_PM_Start']:
                    t_pair[dayTime1].append((max(scheduled_time_filter['WorkTime_PM_Start'], hour1), min(scheduled_time_filter['WorkTime_PM_End'], h2)))
                if hour1 <= scheduled_time_filter['WorkTime_AM_End'] and h2 > scheduled_time_filter['WorkTime_AM_Start']:
                    t_pair[dayTime1].append((max(scheduled_time_filter['WorkTime_AM_Start'], hour1), min(scheduled_time_filter['WorkTime_AM_End'], h2)))
            else:  #leisure time report
                if h2 > scheduled_time_filter['WorkTime_PM_End']:
                    t_pair[dayTime1].append((max(scheduled_time_filter['WorkTime_PM_End']+1, hour1), h2))
                if hour1 < scheduled_time_filter['WorkTime_PM_Start'] and h2 > scheduled_time_filter['WorkTime_AM_End']:
                    t_pair[dayTime1].append((max(scheduled_time_filter['WorkTime_AM_End']+1, hour1), min(scheduled_time_filter['WorkTime_PM_Start']-1, h2)))
                if hour1 < scheduled_time_filter['WorkTime_AM_Start']:
                    t_pair[dayTime1].append((hour1, min(scheduled_time_filter['WorkTime_AM_Start']-1, h2)))
        else:   #leisure day
            if scheduled_time_filter['timeType'] == 2:  #leisure time report
                t_pair[dayTime1].append((hour1, hour2 if dayTime1==dayTime2 else 23))
        
        #move to next day
        date1 = date1 + datetime.timedelta(days = 1)
        hour1 = 0
        tmp_date1 = date1.replace(hour=0)
        dayTime1 = calendar.timegm(tmp_date1.utctimetuple())
        
    logging.debug('Local Time Pair: %r', t_pair)
    return t_pair

def getTimePair(cid,ts1, ts2,scheduled_time_filter, by_day=True):
    zone,_ = get_localzone()
    
    # Normalize scheduled time filter.
    # Before:
    #   timeType: 0,		#0: always, 1: work time, 2: leisure time
    #   WorkTime_Day: ‘0111110’,
    #   WorkTime_AM_Start: ‘0800’,
    #   WorkTime_AM_End: ‘1159’,
    #   WorkTime_PM_Start: ‘1300’,
    #   WorkTime_PM_End: ‘1700’
    # After:
    #   timeType: 0,		#0: always, 1: work time, 2: leisure time
    #   WorkTime_Day: ‘0111110’,
    #   WorkTime_AM_Start: 8,
    #   WorkTime_AM_End: 11,    #The hour of '11' will be included.
    #   WorkTime_PM_Start: 13,
    #   WorkTime_PM_End: 16     #The hour of '16' will be included.

    if scheduled_time_filter.has_key('isNormalized') and scheduled_time_filter['isNormalized']:
        pass
    else:
        scheduled_time_filter['WorkTime_AM_Start'] = int(scheduled_time_filter['WorkTime_AM_Start'][:2]) + (0 if int(scheduled_time_filter['WorkTime_AM_Start'][-2:]) <= 30 else 1)
        scheduled_time_filter['WorkTime_AM_End'] = int(scheduled_time_filter['WorkTime_AM_End'][:2]) + (0 if int(scheduled_time_filter['WorkTime_AM_End'][-2:]) <= 30 else 1) - 1
        scheduled_time_filter['WorkTime_PM_Start'] = int(scheduled_time_filter['WorkTime_PM_Start'][:2]) + (0 if int(scheduled_time_filter['WorkTime_PM_Start'][-2:]) <= 30 else 1)
        scheduled_time_filter['WorkTime_PM_End'] = int(scheduled_time_filter['WorkTime_PM_End'][:2]) + (0 if int(scheduled_time_filter['WorkTime_PM_End'][-2:]) <= 30 else 1) - 1
        scheduled_time_filter['isNormalized'] = True
    
    t_pair = getTimePairLocalZone(cid, ts1, ts2, scheduled_time_filter, zone)
    
    #convert from local zone to UTC
    if by_day:
        result = {}
    else:
        result = []
    
    for ts in t_pair:
        for h1, h2 in t_pair[ts]:
            dt1 = datetime.datetime.fromtimestamp(ts, zone)
            dt1 = dt1.replace(hour=h1)
            dt1 = calendar.timegm(dt1.utctimetuple())
            if by_day:
                h1_utc, d1_utc = getTimestampStartDayTime(cid, dt1)
            dt2 = datetime.datetime.fromtimestamp(ts, zone)
            dt2 = dt2.replace(hour=h2)
            dt2 = calendar.timegm(dt2.utctimetuple())
            if by_day:
                h2_utc, d2_utc = getTimestampStartDayTime(cid, dt2)
                if d1_utc == d2_utc:
                    if not result.has_key(d1_utc):
                        result[d1_utc] = []
                    result[d1_utc].append((h1_utc, h2_utc))
                else:
                    if not result.has_key(d1_utc):
                        result[d1_utc] = []
                    if not result.has_key(d2_utc):
                        result[d2_utc] = []
                    result[d1_utc].append((h1_utc, 23))
                    result[d2_utc].append((0, h2_utc))
            else:
                result.append((dt1, dt2+3600))
                
    logging.debug('Time Pair: %r', result)
    return result