#!/usr/bin/env python
# -*- coding: utf-8 -*-

import gevent
import gevent.pywsgi
import logging
import json
import zlib
import traceback
import base64
import os
import uuid
from gevent_zeromq import zmq
from auth import authentication

from clfcfg import CLFCfg
import log_engine
import log_query
import report
from summary_engine import SummaryEngine
from stats_engine import StatsEngine
from report_manager import ReportManager
from dashboard_manager import DashboardManager
from log_search_engine import LogSearchEngine
import clfutil
import time
import re
import uwsgi
from handler import MsgType

g_inited = False

g_logengine = None
g_sock_main = None
g_report_manager = None
g_dashboard_manager = None
g_sumEngine = SummaryEngine()
g_stats_engine = StatsEngine()
g_log_search_engine = LogSearchEngine()

def get_client_address(environ):
    try:
        return environ['HTTP_X_FORWARDED_FOR'].split(',')[-1].strip()
    except KeyError:
        return environ['REMOTE_ADDR']

def log_upload(environ, start_response):
    # Cloud mode should not use this service.
    logging.debug('log_upload: ')

    wsgi_input = environ['wsgi.input']
    content_length = int(environ.get('CONTENT_LENGTH', -1))
    try:
        reqdata = wsgi_input.read(content_length)
        logging.debug('req: %r', reqdata)
        req = json.loads(reqdata)

        location = req.pop('location', '')
        ts = req.pop('ts')
        zip = req.pop('zip', 0)
        content = req.pop('content', '')

        client_ip = get_client_address(environ)
        #save content
        if len(content) > 0:
            # Standalone mode. Save data to local.
            if zip == 1:
                data = zlib.decompress(base64.decodestring(content))
            else:
                data = content
            client_ip = get_client_address(environ)
            file_name = '%d_%s_%s.log' % (ts, client_ip, uuid.uuid1())
            location = g_logengine.saveRawLog(file_name, data)

        # send message to main processor
        msg = {'ts': ts,
               'ip': client_ip,
               'location': location}
        logging.debug('Send message to main: ' + json.dumps(msg))
        uwsgi.lock()
        g_sock_main.send_pyobj({'type': MsgType['DATA'], 'data':msg})
        uwsgi.unlock()
        time.sleep(0.5)

        #response the client
        start_response('200 OK', [('Content-Type', 'application/json'), ('Connection', 'close')])
        return json.dumps({'result': 0})
    except Exception as e:
        logging.exception('log_upload failed: %r', e)
        start_response('500 Internal Server Error', [('Content-Type', 'text/plain')])
        return traceback.format_exc()


def log_page_query(environ, start_response):
    logging.debug('log_page_query: ')

    wsgi_input = environ['wsgi.input']
    content_length = int(environ.get('CONTENT_LENGTH', -1))
    try:
        req = json.loads(wsgi_input.read(content_length))

        logging.debug(req)
        query = req.pop('query', {})
        paging = req.pop('paging', {})
        column = req.pop('column', '')
        cursor = req.pop('cursor', '')
        if req.has_key('session_id'):
            session_id = req.pop('session_id')
        else:
            session_id = 0
        #print 'service session id  :%r'%(session_id)
        #if re.match('\d+B$', count):
        #    count = 10
        #else:
        #    count =  int(count)

        if column:
            column = column.split(',')

        start_response('200 OK', [('Content-Type', 'application/json'), ('Connection', 'close')])
        return log_query.queryLogs(query[CLFCfg.CLF_CompanyID], column, query['filters'], query['timeBegin'], query['timeEnd'], paging['skip'], paging['limit'], cursor,session_id,query['reportTemplate'], query['scheduledTime'] if query.has_key('scheduledTime') else None)

    except Exception as e:
        logging.exception('log_page_query failed:%s', e)
        start_response('500 Internal Server Error', [('Content-Type', 'text/plain')])
        return traceback.format_exc()

def log_stream_query(environ, start_response):
    logging.debug('log_stream_query: ')

    wsgi_input = environ['wsgi.input']
    content_length = int(environ.get('CONTENT_LENGTH', -1))
    try:
        req = json.loads(wsgi_input.read(content_length))
        logging.debug(req)

        query = req.pop('query', {})
        column = req.pop('column', '')

        if column:
            column = column.split(',')

        start_response('200 OK', [('Content-Type', 'application/json')])

        log_generator = log_query.queryLogsStream(query[CLFCfg.CLF_CompanyID], column, query['filters'], query['timeBegin'], query['timeEnd'], query['reportTemplate'],query['reportTemplate'], query['scheduledTime'] if query.has_key('scheduledTime') else None)

        for logs in log_generator:
            yield logs

    except Exception as e:
        logging.exception('log_stream_query failed:%s', e)
        start_response('500 Internal Server Error', [('Content-Type', 'text/plain')])
        yield traceback.format_exc()

def report_query(environ, start_response):
    logging.debug('report_query: ')
    wsgi_input = environ['wsgi.input']
    content_length = int(environ.get('CONTENT_LENGTH', -1))
    try:
        req = json.loads(wsgi_input.read(content_length))
        rep_param = req.pop('reportParam', {})

        start_response('200 OK', [('Content-Type', 'application/json')])
        if CLFCfg.CLF_STATS_DB_TYPE == CLFCfg.CLF_DB_STATS_SQLITE:
            rep = g_report_manager.build_reportEx(rep_param['reportTemplate'],
                                    rep_param['cid'],
                                    rep_param['timeBegin'],
                                    rep_param['timeEnd'],
                                    rep_param['filters'],
                                    rep_param['LDAPGroups'] if rep_param.has_key('LDAPGroups') else [],
                                    rep_param['IPRanges'] if rep_param.has_key('IPRanges') else [],
                                    rep_param['scheduledTime'] if rep_param.has_key('scheduledTime') else None)
        else:
            rep = g_report_manager.build_report(rep_param['reportTemplate'],
                                    rep_param['cid'],
                                    rep_param['timeBegin'],
                                    rep_param['timeEnd'],
                                    rep_param['filters'],
                                    rep_param['LDAPGroups'] if rep_param.has_key('LDAPGroups') else [],
                                    rep_param['IPRanges'] if rep_param.has_key('IPRanges') else [],
                                    rep_param['scheduledTime'] if rep_param.has_key('scheduledTime') else None)

        topN = 10
        if 'topN' in rep_param.keys():
            topN = rep_param['topN']
        return json.dumps(rep.get_data(topN))
    except Exception as e:
        logging.exception('report_query failed:%s', e)
        start_response('500 Internal Server Error', [('Content-Type', 'text/plain')])
        return traceback.format_exc()

def dashboard_query(environ, start_response):
    logging.debug('dashboard_query: ')
    wsgi_input = environ['wsgi.input']
    content_length = int(environ.get('CONTENT_LENGTH', -1))
    try:
        req = json.loads(wsgi_input.read(content_length))
        rep_param = req.pop('reportParam', {})

        start_response('200 OK', [('Content-Type', 'application/json')])
        rep = g_dashboard_manager.build_dashboard(rep_param['reportTemplate'],
                                    rep_param['cid'],
                                    rep_param['timeBegin'],
                                    rep_param['timeEnd'],
                                    rep_param['filters'])

        topN = 10
        if 'topN' in rep_param.keys():
            topN = rep_param['topN']
        return json.dumps(rep.get_data(topN))
    except Exception as e:
        logging.exception('dashboard_query failed:%s', e)
        start_response('500 Internal Server Error', [('Content-Type', 'text/plain')])
        return traceback.format_exc()

def log_purge(environ, start_response):
    '''
        req = {
                'purgeSummary': 1,
                'purgeStats': 1,
                'ts': ts,
                'cid': cid
              }
    '''
    logging.debug('log_purge: ')
    wsgi_input = environ['wsgi.input']
    content_length = int(environ.get('CONTENT_LENGTH', -1))
    try:
        req = json.loads(wsgi_input.read(content_length))
        logging.debug(req)
        cid = req['cid']
        ts = req['ts']
        if req['purgeSummary'] == 1:
            if ts < 0:
                ts_ = g_logengine.getLogBlockStartTs(cid)
                ts = (ts_/3600/24 - ts)*3600*24
            g_sumEngine.removeSummaries(cid, ts)
            g_logengine.removeLogBlocks(cid, ts)

        if req['purgeStats'] == 1:
            if ts < 0:
                statsTs = g_stats_engine.findStats2(
                        [{'col': CLFCfg.CLF_CompanyID, 'op': 'eq', 'value': cid}],
                        {CLFCfg.CLF_TimeStamp: 1})
                ts_list = []
                for s in statsTs:
                    ts_list.append(s['CLFCfg.CLF_TimeStamp'])
                ts_ = ts_list.sort()[0]
                ts = (ts_/3600/24 - ts)*3600*24
            g_stats_engine.cleanupStats(cid, ts)

        #purge dashboard stats
        dash_keep = CLFCfg.config.getint('log_purge','dashboard_stats_keep')
        dash_ts = int(time.time())/3600/24 - dash_keep
        dash_ts = dash_ts * 3600 * 24 + time.timezone
        logging.debug('Purge dashboard stats before: %s', time.strftime('%Y/%m/%d', time.localtime(dash_ts)))
        g_stats_engine.cleanupDashboardStats(cid, dash_ts)

        start_response('200 OK', [('Content-Type', 'application/json')])
        return ''
    except Exception as e:
        logging.exception('log_purge failed:%s', e)
        start_response('500 Internal Server Error', [('Content-Type', 'text/plain')])
        return traceback.format_exc()

def log_size(environ, start_response):
    logging.debug('log_size: ')
    try:
        resp = {}
        resp['raw_size'] = g_logengine.getSize()
        resp['sumdb_size'] = g_sumEngine.getSize()
        if CLFCfg.CLF_SUMMARY_DB_TYPE != CLFCfg.CLF_STATS_DB_TYPE:
            resp['statsdb_size'] = g_stats_engine.getSize()
        else:
            resp['statsdb_size'] = 0
        start_response('200 OK', [('Content-Type', 'application/json')])
        return json.dumps(resp)
    except Exception as e:
        logging.exception('log_size failed:%s', e)
        start_response('500 Internal Server Error', [('Content-Type', 'text/plain')])
        return traceback.format_exc()

def log_search(environ, start_response):
    logging.debug('log_search: ')
    wsgi_input = environ['wsgi.input']
    content_length = int(environ.get('CONTENT_LENGTH', -1))
    try:
        req = json.loads(wsgi_input.read(content_length))

        start_response('200 OK', [('Content-Type', 'application/json')])
        resp = g_log_search_engine.findKeyword(req['column'], req['keyword'])

        return json.dumps(resp)
    except Exception as e:
        logging.exception('log_search failed:%s', e)
        start_response('500 Internal Server Error', [('Content-Type', 'text/plain')])
        return traceback.format_exc()

@authentication
def application(environ, start_response):

    global g_inited
    if not g_inited:
        init()

    path_info = environ['PATH_INFO']

    if (path_info == '/log_upload'):
        return log_upload(environ, start_response)
    elif (path_info == '/log_page_query'):
        return log_page_query(environ, start_response)
    elif (path_info == '/log_stream_query'):
        return log_stream_query(environ, start_response)
    elif (path_info == '/report_query'):
        return report_query(environ, start_response)
    elif (path_info == '/dashboard_query'):
        return dashboard_query(environ, start_response)
    elif (path_info == '/log_purge'):
        return log_purge(environ, start_response)
    elif (path_info == '/log_size'):
        return log_size(environ, start_response)
    elif (path_info == '/log_search'):
        return log_search(environ, start_response)

def init():
    global g_inited
    if g_inited:
        return

    print 'Init...'
    CLFCfg.init()
    global g_logengine
    g_logengine = log_engine.LogEngine()

    global g_sock_main
    zmq_ctx = zmq.Context()
    g_sock_main = zmq_ctx.socket(zmq.PUSH)
    g_sock_main.connect(CLFCfg.CLF_MAIN_PRE_ADDR)

    global g_report_manager
    g_report_manager = ReportManager()

    global g_dashboard_manager
    g_dashboard_manager = DashboardManager()
    g_inited = True


